#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
#
# These classes generate view data for clusters.

from desktop.lib.django_util import render_to_string

from health import query
from health.models import GMETAD, DATA_NODE, NAME_NODE, JOB_TRACKER, TASK_TRACKER
from health.data import DataPoint

class AbstractClusterView(object):
  """
  Cluster views are initialized with datapoints that are relevant
  to them, and then can render summary and participant strings.
  """
  def __init__(self, data, cluster):
    self.data = data
    self.cluster = cluster

  def render_summary(self):
    return render_to_string(self.summary_template, dict(summary=self.summary()))

  def render_participants(self):
    return render_to_string(self.participant_template, dict(participants=self.participants()))

class GangliaClusterView(AbstractClusterView):
  summary_template = "ganglia_summary.mako"
  participant_template = "ganglia_participants.mako"

  def summary(self):
    # First do some aggregation:
    GANGLIA_KEYS = {
      "mem_free": sum,
      "mem_total": sum,
      "load_one": query.average,
      "load_five": query.average,
      "load_fifteen": query.average,
    }
    acc = {}
    for row in self.data:
      if row.key in GANGLIA_KEYS.keys():
        acc.setdefault(row.key, []).append(float(row.value["VAL"]))
    for key, f in GANGLIA_KEYS.items():
      acc[key] = f(acc.get(key, []))

    # Now prepare the summary
    summary = {}
    # Ganglia measures in KB
    summary["memory_used"] = 1024*(acc["mem_total"] - acc["mem_free"])
    summary["memory_total"] = 1024*(acc["mem_total"])
    for i in ["load_one", "load_five", "load_fifteen"]:
      summary[i] = acc[i]
      
    summary["hosts"] = query.bucket_count(filter(matches_role(GMETAD), self.data), DataPoint.STATE, DataPoint.STATES)

    summary["link"] = self.cluster.url + "?c=unspecified"
    return summary

  def participants(self):
    GANGLIA_MAPPING = {
      "load_one": "load_one",
      "load_five": "load_five",
      "load_fifteen": "load_fifteen",
      "bytes_in": "kb_in",
      "bytes_out": "kb_out",
      "state": "state",
      "proc_run": "running_processes",
    }
    def extract(grouping, dps):
      m = query.remap(dps, GANGLIA_MAPPING)
      for key, value in m.items():
        if key != "state":
          m[key] = float(value["VAL"])
      # Convert to KB
      m["kb_in"] = m.get("kb_in", 0) / 1024
      m["kb_out"] = m.get("kb_out", 0) / 1024
      # This is always an integer
      if m.get("running_processes") is not None:
        m["running_processes"] = int(m.get("running_processes"))

      m["host"] = grouping.id
      m["summary_link"] = self.cluster.url + "?c=unspecified&h=" + grouping.id
      return m

    hosts = query.query(data=self.data,
      select=extract,
      where=matches_role(GMETAD),
      groupby=lambda row: row.participant)

    return hosts

class HDFSClusterView(AbstractClusterView):
  summary_template = "hdfs_summary.mako"
  participant_template = "hdfs_participants.mako"

  def summary(self):
    # Extract the summary information.
    SUMMARY_KEY_REMAPPING = {
      "dfs_health_report:bytesTotal": "capacity",
      "dfs_health_report:bytesNonDfs": "non_dfs",
      "dfs_health_report:bytesRemaining": "dfs_remaining"
    }
    summary = query.remap(
      [ dp for dp in self.data if dp.participant is None ],
      SUMMARY_KEY_REMAPPING)

    summary["datanodes"] = query.bucket_count(filter(matches_role(DATA_NODE), self.data), DataPoint.STATE, DataPoint.STATES)
    dp = find_single_participant_datapoint(
        data=self.data,
        role=NAME_NODE, 
        key=DataPoint.STATE)
    if not dp:
      # TODO(todd) can this really exist?
      summary["namenode"] = None
      summary["namenode_state"] = DataPoint.STATE_DOWN[1]
    else:
      summary["namenode"] = dp.participant
      summary["namenode_state"] = dp.value
    return summary

  def participants(self):
    # Add in the datanode data
    DATANODE_MAPPING = {
      "datanode_report:dfsUsed": "used",
      "datanode_report:capacity": "capacity",
      "state": "state",
    }
    def extract(grouping, dps):
      dn = dict()
      dn["host"] = grouping.ip
      dn['participant'] = grouping
      dn.update(query.remap(dps, DATANODE_MAPPING))
      return dn
    datanodes = query.query(data=self.data,
      select=extract,
      where=matches_role(DATA_NODE),
      groupby=lambda row: row.participant)

    return datanodes

class MRClusterView(AbstractClusterView):
  summary_template = "mr_summary.mako"
  participant_template = "mr_participants.mako"

  def summary(self):
    d = dict()
    dp = find_single_participant_datapoint(
        data=self.data,
        role=JOB_TRACKER, 
        key=DataPoint.STATE)
    if not dp:
      d["jobtracker_state"] = DataPoint.STATE_DOWN[1]
    else:
      d["jobtracker_state"] = dp.value
    SUMMARY_KEY_REMAPPING = {
      "mr_cluster_status:mapTasks": "maps",
      "mr_cluster_status:maxMapTasks": "map_capacity",
      "mr_cluster_status:reduceTasks": "reduces",
      "mr_cluster_status:maxReduceTasks": "reduce_capacity",
      "mr_cluster_status:totalSubmissions": "submissions",
    }
    d.update(query.remap(self.data, SUMMARY_KEY_REMAPPING))
    d["tasktrackers"] = per_state_count(self.data, TASK_TRACKER)
    d['jobtracker'] = dp.participant
    return d

  def participants(self):
    TASK_TRACKER_MAPPING = {
      "mr_tasktracker:httpPort": "http_port",
      "mr_tasktracker:host": "host",
      "mr_tasktracker:mapCount": "maps",
      "mr_tasktracker:maxMapTasks": "map_capacity",
      "mr_tasktracker:reduceCount": "reduces",
      "mr_tasktracker:maxReduceTasks": "reduce_capacity",
      "state": "state",
    }
    def extract(grouping, dps):
      d = query.remap(dps, TASK_TRACKER_MAPPING)
      d["host"] = grouping.ip
      d["participant"] = grouping
      return d
    datanodes = query.query(data=self.data,
      select=extract,
      where=matches_role(TASK_TRACKER),
      groupby=lambda row: row.participant)

    return datanodes

# Utility methods for common aggregations.
def matches_role(role):
  """Returns a function that matches a role."""
  def f(dp):
    return dp.participant is not None and dp.participant.role == role
  return f

def find_single_participant_datapoint(data, role, key):
  def is_interesting_datapoint(dp):
    return dp.participant is not None \
      and dp.participant.role == role \
      and dp.key == key
  return query.find_single(data=data,
    select=lambda dp: dp,
    where=is_interesting_datapoint,
    default=None)

def find_single_participant_datapoint_value(data, role, key, default):
  """Finds a single value matching a role."""
  dp = find_single_participant_datapoint(data, role, key)
  if not dp:
    return default
  else:
    return dp.value

def per_state_count(data, role=None):
  if role:
    data = filter(matches_role(role), data)
  return query.bucket_count(data, DataPoint.STATE, DataPoint.STATES)
