#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
#
# Classes to retrieve information about clusters.

import logging
import socket
import hadoop.cluster
from thrift.transport.TTransport import TTransportException

from desktop.lib import thrift_util, django_util
from health.data import DataPoint
from health.models import NAME_NODE, DATA_NODE, JOB_TRACKER, TASK_TRACKER, CLUSTER_TYPES, HDFS, MR, GANGLIA, Participant

from hadoop.api.hdfs.ttypes import DatanodeReportType


def resolveip(host):
  """
  Wrapper around socket.gethostbyname that deals with an Ubuntu quirk.
  Ubuntu ships with an /etc/hosts entry that causes some localhost
  lookups to resolve to 127.0.1.1 instead of 127.0.0.1, which makes
  pseudo-distributed clusters look like two hosts. This is ugly,
  but makes health make a lot more sense for new users, and should
  be harmless
  """
  ret = socket.gethostbyname(host)
  if ret == "127.0.1.1":
    return "127.0.0.1"
  return ret

def id_from_ip_port(host, port):
  """
  Constructs string from host and port.

  It's problematic to use IPs as unique keys for machines, but
  there's nothing easily accessible that's better.
  """
  return "%s:%d" % (resolveip(host), port)

class Cluster(object):
  """
  A cluster is meant to represent a coherent grouping of machines,
  which typically provide a service.  The DFS daemons, all together,
  form a cluster; the MR daemons form a separate cluster.  Monitoring
  software may also form a cluster.

  Clusters are read from configuration, so they're not a Model,
  but they act a little bit like one sometimes.

  Clusters are also responsible for emitting DataPoints on
  get_fresh_datapoints().
  """
  def __init__(self, type, name):
    self.name = name
    self.type = type
    assert self.type in CLUSTER_TYPES

  # Map of (type, name) tuples into Cluster objects.
  CLUSTERS = None

  @classmethod
  def get_cluster(cls, type, name):
    cls._initialize_all()
    return cls.CLUSTERS[ (type, name) ]


  @classmethod
  def all(cls):
    cls._initialize_all()
    return cls.CLUSTERS.values()

  @classmethod
  def _initialize_all(cls):
    if cls.CLUSTERS is not None:
      # Already initialized
      return

    cls.CLUSTERS = {}
    cls._initialize_hdfs()
    cls._initialize_mr()
    cls._initialize_ganglia()

  @classmethod
  def _initialize_hdfs(cls):
    for name, filesystem in hadoop.cluster.get_all_hdfs().items():
      cls.CLUSTERS[(HDFS, name)] = HDFSCluster(name, filesystem)

  @classmethod
  def _initialize_mr(cls):
    for name, mrcluster in hadoop.cluster.all_mrclusters().items():
      cls.CLUSTERS[(MR, name)] = MRCluster(name, mrcluster)

  @classmethod
  def _initialize_ganglia(cls):
    from health import ganglia_cluster
    for name, gangliacluster in ganglia_cluster.all_ganglias().items():
      cls.CLUSTERS[(GANGLIA, name)] = gangliacluster

  def __str__(self):
    return "Cluster(type=%s, name=%s)" % (self.type, self.name)

  @property
  def id(self):
    """
    Returns unique id for the cluster.

    It's convenient to have ids, so that you can refer to a cluster
    by one thing instead of two.  We just join the type and name
    with a slash.
    """
    return "%s/%s" % (self.type, self.name)

  @classmethod
  def get_by_id(cls, id):
    """Inverse of id, for retrieval."""
    type, name = id.split("/", 2)
    return cls.get_cluster(type, name)

  def get_fresh_datapoints(self):
    """
    Returns an iterator of facts about this cluster.
    """
    return []

  def to_jsonable(self):
    return dict(obj_type='CLUSTER',
                name=self.name,
                type=self.type)


class HDFSCluster(Cluster):
  """Queries HDFS for data."""
  def __init__(self, name, fs):
    super(HDFSCluster, self).__init__(HDFS, name)
    self.fs = fs
    # Figure out if we have a local or HDFS file system.
    if hasattr(fs, "nn_client"):
      self.namenode = fs.nn_client
      self.fs = fs
    else:
      self.namenode = None

  def get_fresh_datapoints(self):
    # Ignore unless we have an HDFS file system.
    if self.namenode is None:
      return

    nn_ip = resolveip(self.fs.host)
    nn_participant = coalesce_master_participant(self, NAME_NODE,
      dict(
        id=id_from_ip_port(nn_ip, self.fs.thrift_port),
        ip=nn_ip,
        thrift_port=self.fs.thrift_port,
        name=self.fs.uri))

    def namenode_state(health_report):
      """
      Used twice to determine state of namenode.
      If namenode is down, HDFS is unusable, so it has
      a special place.
      """
      if not health_report:
        return DataPoint(self, nn_participant, *DataPoint.STATE_DOWN)
      else:
        django_util.update_if_dirty(nn_participant,
                                    http_port=health_report.httpPort)
        return DataPoint(self, nn_participant, *DataPoint.STATE_UP)

    # RPCs to NameNode.
    # TODO(philip): It's impossible to tell from DatanodeReport
    # whether a datanode is dead or alive, so we do an extra RPC
    # for that.
    try:
      health_report = self.namenode.getHealthReport(None)
      runtime_info = self.namenode.getRuntimeInfo(None)
      live_datanode_report = self.namenode.getDatanodeReport(None, DatanodeReportType.LIVE_DATANODES)
      dead_datanode_report = self.namenode.getDatanodeReport(None, DatanodeReportType.DEAD_DATANODES)
      for x in live_datanode_report:
        x.alive = True
      for x in dead_datanode_report:
        x.alive = False
      datanode_report = live_datanode_report + dead_datanode_report
    except TTransportException, e:
      # We failed to communicate, so we'll simply mark the namenode as down.
      yield namenode_state(None)
      # Short-circuit everything else.
      return

    # Datanodes:
    def report_to_participant(r):
      """
      Converts a health report into a dict suitable for being
      a Participant
      """
      return dict(id=r.storageID,
          ip=resolveip(r.host),
          thrift_port=r.thriftPort,
          http_port=r.httpPort,
          name=r.name)
    for participant, report in coalesce_participants(
        self,
        DATA_NODE,
        map(report_to_participant, datanode_report),
        datanode_report):
      if report is None or not report.alive:
        state = DataPoint.STATE_DOWN
      else:
        state = DataPoint.STATE_UP
      yield DataPoint(self, participant, *state)

      if report:
        for key, value in thrift_util.thrift2json(report).items():
          yield DataPoint(self, participant, "datanode_report:%s" % key, value)

    # Namenode:
    yield namenode_state(health_report)
    for key, value in thrift_util.thrift2json(health_report).items():
      # We skip upgradeStatus, since it's nested.
      if key != "upgradeStatus":
        yield DataPoint(self, None, "dfs_health_report:%s" % key, value)

    # NameNode heap usage
    for key, value in thrift_util.thrift2json(runtime_info).items():
      yield DataPoint(self, nn_participant, "runtime_info:%s" % key, value)


class MRCluster(Cluster):
  """Manages querying from JT"""
  def __init__(self, name, mrcluster):
    super(MRCluster, self).__init__(MR, name)
    self.jt = mrcluster

  def get_fresh_datapoints(self):
    jt_ip = resolveip(self.jt.host)
    jt_participant = coalesce_master_participant(self, JOB_TRACKER,
      dict(
        id=id_from_ip_port(jt_ip, self.jt.thrift_port),
        ip=jt_ip,
        thrift_port=self.jt.thrift_port,
        # TODO(philip): May want a better name.
        name="JobTracker: %s" % jt_ip))

    def jt_state(cluster_status):
      """Returns state up/down for JT"""
      if not cluster_status:
        return DataPoint(self, jt_participant, *DataPoint.STATE_DOWN)
      else:
        return DataPoint(self, jt_participant, *DataPoint.STATE_UP)

    try:
      trackers = self.jt.all_task_trackers().trackers
      cluster_status = self.jt.cluster_status()
      # Now that we have cluster status, we can fill in the correct HTTP port
      django_util.update_if_dirty(jt_participant, http_port=cluster_status.httpPort)
      runtime_info = self.jt.runtime_info()
    except TTransportException, e:
      yield jt_state(None)
      return

    active = set(cluster_status.activeTrackerNames)
    blacklisted = set(cluster_status.blacklistedTrackerNames)

    # Jobtracker
    yield jt_state(cluster_status)
    for key, value in thrift_util.thrift2json(cluster_status).items():
      yield DataPoint(self, None, "mr_cluster_status:%s" % key, value)

    # Jobtracker heap usage
    for key, value in thrift_util.thrift2json(runtime_info).items():
      yield DataPoint(self, jt_participant, "runtime_info:%s" % key, value)

    def get_tt_ip(tracker):
      return resolveip(tracker.host)

    # TaskTrackers
    def m(tracker):
      return dict(
            id=id_from_ip_port(tracker.host, tracker.httpPort),
            ip=get_tt_ip(tracker),
            thrift_port=None,
            http_port=tracker.httpPort,
            name=tracker.trackerName)

    coalesced_tts = coalesce_participants(self, TASK_TRACKER,
      map(m, trackers), trackers)
    for obj, tracker in coalesced_tts:
      if not tracker:
        yield DataPoint(self, obj, *DataPoint.STATE_DOWN)
      elif tracker.trackerName in active:
        yield DataPoint(self, obj, *DataPoint.STATE_UP)
      elif tracker.trackerName in blacklisted:
        yield DataPoint(self, obj, *DataPoint.STATE_ERROR)
      else:
        # This could happen in a race between the two thrift calls, say.
        logging.error("Unexpected tracker: %s" % tracker)
        yield DataPoint(self, obj, *DataPoint.STATE_DOWN)

      if tracker:
        for key, value in thrift_util.thrift2json(tracker).items():
          yield DataPoint(self, obj, "mr_tasktracker:%s" % key, value)

def coalesce_master_participant(cluster, role, participant, join_data=None):
  """
  Conveniently grabs the 'master' node.
  """
  coalesced = coalesce_participants(cluster, role, [participant], [join_data])
  # It's possible we had a prior life of a master participant (eg it has changed ports).
  # In this case we want to remove the ones that don't exist anymore
  for other_participant, data in coalesced:
    if other_participant.id != participant['id']:
      other_participant.delete()
    else:
      toret_participant = other_participant

  return toret_participant

def coalesce_participants(cluster, role, participant_data, join_data):
  """
  Performs an outer join (via hash) between the Participant model and
  the arguments, where participant_data.id is the join_key,
  and join_data is returned alongside with the participant object.
  Also (side-effect!), updates the Participant model with any
  newly-seen participants.

  @param cluster The relevant cluster in which to find participants
  @param role The role type
  @param participant_data A list containing dictionaries, whose keys
  work with the Participant model, from which to populate the participants.
  @param join_data Data associated with participant_data.  Should be same length
  as participant_data.
  @returns A list of (Participant, join_data_element) pairs.
  """
  assert len(participant_data) == len(join_data)

  by_id = {}
  for participant in Participant.objects.filter(cluster_type=cluster.type, cluster_name=cluster.name, role=role):
    by_id[participant.id] = (participant, None)

  for p_data, j_data in zip(participant_data, join_data):
    val = by_id.get(p_data["id"])
    if val is None:
      logging.info("Saving new participant: %s, %s" % (str(p_data), str(j_data)))
      p = Participant(cluster_type=cluster.type, cluster_name=cluster.name, role=role, **p_data)
      p.save()
    else:
      p, _ = val
      assert p.cluster_name == cluster.name
      assert p.cluster_type == cluster.type
      assert p.role == role
      assert p.id == p_data["id"]
      django_util.update_if_dirty(p, **p_data)
    by_id[p.id] = (p, j_data)

  return by_id.values()
