#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
#
# Representations of instantaneous health data in cluster health.
# models.py contains persisted data.

class DataPoint(object):
  """
  Components of the health app generate DataPoints, which
  are associated with a cluster, or a cluster-participant pair.

  Views will query these data points (often doing
  grouping and aggregation in the process), and present
  the results of those queries to the user.

  These points are not currently persisted, but it would
  be easy enough to log or persist them appropriately.
  """

  # Some common data points:
  STATE = "state"
  STATE_UP = ("state", "UP")
  STATE_ERROR = ("state", "ERROR")
  STATE_DOWN = ("state", "DOWN")
  STATES = ("UP", "ERROR", "DOWN")

  # Used for the comparison:
  _state_value_map = { STATE_UP:3, STATE_ERROR:2, STATE_DOWN:1 }
  @classmethod
  def state_key(cls, a):
    """Extracts a comparision key from a state.
    This is used by min() or sort().
    States that are more serious sort as lower numbers.
    See http://docs.python.org/library/stdtypes.html#mutable-sequence-types
    """
    return cls._state_value_map[a]

  def __init__(self, cluster=None, participant=None, key=None, value=None):
    """
    Cluster and participant are optional, but are expected to be from the 
    Cluster and Participant models.  If participant is specified, cluster should be too.
    Keys should not be None, but values have a tendency to want to be None sometimes,
    and that's ok, if not preferred.
    """
    if participant:
      assert cluster
    assert key
    self.cluster = cluster
    self.participant = participant
    self.key = key
    self.value = value

  def to_jsonable(self):
    return self.__dict__

  @property
  def key_value(self):
    return self.key, self.value

  def __str__(self):
    if self.cluster is not None:
      c = self.cluster.id
    else:
      c = None
    return "DP<%s, %s, %s, %s>" % (c, self.participant, self.key, self.value)
