#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.

import logging


class Extractor(object):
  """
  An Extractor is responsible for taking all of the datapoints
  for a given participant and extracting a numeric suitable for
  display in a report or heatmap.

  This is an abstract class - implementors should override
  the extract method.
  """
  def __init__(self,
               name,
               heat_detector=None,
               formatter=str):
    """
    @param name: A user-understandable name for the metric (eg "Number of failed tasks")
    @param heat_detector: a HeatDetector subclass used to pick colors for this measurement in a heat map
    @param formatter: a callbale that takes the extracted value and makes it suitable for user display
    """
    self.name = name
    self.heat_detector = heat_detector
    self.formatter = formatter

  def extract(self, participant_data):
    """Extract a numeric measurement from all of the datapoints for a
    participant.

    @param participant_data: a dictionary of DataPoints keyed by the DataPoint.key
    """
    raise Exception("Use of abstract class!")

class SimpleExtractor(Extractor):
  """Extract the value of a given datapoint for a participant by key.
  If the value exists, runs it through a specified transform function."""

  def __init__(self,
               name,
               key,
               transform=lambda x:x,
               **kwargs):
    """
    @param transform: the function through which the value will be run before extraction
    """
    super(SimpleExtractor, self).__init__(name, **kwargs)
    self.transform = transform
    self.key = key

  def extract(self, data):
    val = data.get(self.key)
    if val is not None:
      return self.transform(val)
    else:
      return None

class IntExtractor(SimpleExtractor):
  def __init__(self, name, key, **kwargs):
    super(IntExtractor, self).__init__(name, key, transform=int, **kwargs)

class MetricExtractor(Extractor):
  def __init__(self,
               name,
               metrics_context,
               key,
               **kwargs):
    super(MetricExtractor, self).__init__(name, **kwargs)
    self.metrics_context = metrics_context
    self.key = key

  def extract(self, data):
    metrics_container = data.get('metrics:%s' % self.metrics_context)
    if not metrics_container:
      return None
    if not isinstance(metrics_container, list) or \
          len(metrics_container) != 1 or \
          len(metrics_container[0]) != 2:
      logging.warn("Bad metrics container: %s" % repr(metrics_container))
      return None

    metrics_tag, metrics = metrics_container[0]
    return metrics.get(self.key)

class PercentageExtractor(Extractor):
  """Extract a ratio between two other extractors"""
  def __init__(self, name,
               numerator_extractor, denominator_extractor,
               **kwargs):
    super(PercentageExtractor, self).__init__(name, **kwargs)

    self.numerator_extractor = numerator_extractor
    self.denominator_extractor = denominator_extractor

  def extract(self, participant_data):
    num = self.numerator_extractor.extract(participant_data)
    denom = self.denominator_extractor.extract(participant_data)

    if num is None or denom is None:
      return None

    if denom == 0:
      logging.warn("Denominator 0 for extractor: " + self.name)
      return None

    return float(num) / denom
