#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
"""
Library of extractors specific to the different services in Hadoop.
"""

import time

from health.extractor_types import *
from health.heat_detectors import *
from health.formatters import *

def transform_state(state):
  if state == 'DOWN':
    return 0
  elif state == 'UP':
    return 1
  else:
    raise Exception("Bad state: %s" % state)

def format_state(state):
  if state == 0:
    return "DOWN"
  else:
    return "UP"

def transform_absolute_relative_millis(millis):
  """
  The tasktracker reports its heartbeat timestamp
  in millis since the epoch. We'd really like a time
  relative to now, so we transform it.
  """
  return time.time()*1000 - int(millis)

node_up = SimpleExtractor(
  "Daemon Status",
  "state",
  transform=transform_state,
  heat_detector=ThresholdDetector(0.4, 0.5, None, None),
  formatter=format_state)

# JVM metrics
threads_blocked = MetricExtractor(
  "Threads Blocked",
  "jvm.metrics",
  "threadsBlocked",
  heat_detector=HighBadDetector())

jvm_heap_used = MetricExtractor(
  "Memory Used",
  "jvm.metrics",
  "memHeapCommittedM",
  formatter=format_size_mb)

jvm_heap_max = MetricExtractor(
  "Max Memory",
  "jvm.metrics",
  "maxMemoryM",
  formatter=format_size_mb)

jvm_percent_heap_used = PercentageExtractor(
  "% Heap Used",
  jvm_heap_used,
  jvm_heap_max,
  heat_detector=ThresholdDetector(None, None, 0.8, 0.93),
  formatter=format_percent_fraction)

# Task Tracker stuff
mr_failure_count = IntExtractor(
  "Failure Count",
  "mr_tasktracker:failureCount",
  heat_detector=HighBadDetector())

tt_last_seen = SimpleExtractor(
  "Last Heartbeat",
  "mr_tasktracker:lastSeen",
  transform=transform_absolute_relative_millis,
  formatter=format_millis_ago,
  heat_detector=ThresholdDetector(None, None, 4000, 60000))

mr_running_maps = MetricExtractor(
  "Map Tasks Running",
  "mapred.tasktracker",
  "maps_running",
  heat_detector=HighBadDetector())

mr_running_reduces = MetricExtractor(
  "Reduce Tasks Running",
  "mapred.tasktracker",
  "reduces_running",
  heat_detector=AbnormalBadDetector())

# RPC stuff
rpc_open_connections = MetricExtractor(
  "Open RPC Connections",
  "rpc.metrics",
  "NumOpenConnections",
  heat_detector=HighBadDetector())

rpc_callqueue_len = MetricExtractor(
  "RPC Call Queue Length",
  "rpc.metrics",
  "callQueueLen",
  heat_detector=HighBadDetector())

# DN stuff
dn_last_seen = IntExtractor(
  "Last Heartbeat",
  "datanode_report:millisSinceUpdate",
  heat_detector=ThresholdDetector(None, None, 4000, 40000),
  formatter=format_millis_ago)

dn_remaining_space = IntExtractor(
  "Remaining space",
  "datanode_report:remaining")

dn_capacity = IntExtractor(
  "Capacity",
  "datanode_report:capacity")

dn_avg_blockreport_time = MetricExtractor(
  "Average Block Report Time (ms)",
  "dfs.datanode",
  "blockReports_avg_time",
  heat_detector=HighBadDetector(),
  formatter=format_millis)

dn_block_verification_failures = MetricExtractor(
  "Block Verification Failures",
  "dfs.datanode",
  "block_verification_failures",
  heat_detector=HighBadDetector())

dn_disk_space = PercentageExtractor(
  "% Disk Remaining",
  dn_remaining_space, dn_capacity,
  heat_detector=ThresholdDetector(0.03, 0.20, None, None),
  formatter=format_percent_fraction)

# NN stuff
nn_fsimage_load_time = MetricExtractor(
  "FSImage Load Time",
  "dfs.namenode",
  "fsImageLoadTime",
  formatter=format_millis)

nn_under_replicated_blocks = MetricExtractor(
  "Under-replicated Blocks",
  "dfs.FSNamesystem",
  "UnderReplicatedBlocks")

nn_corrupt_blocks = MetricExtractor(
  "Corrupt Blocks",
  "dfs.FSNamesystem",
  "CorruptBlocks")

nn_num_files = MetricExtractor(
  "Number of Files",
  "dfs.FSNamesystem",
  "FilesTotal")



def get_heatmap_extractors():
  """Return the extractors that should be useful to display in a heatmap."""
  return [
    node_up,
    jvm_percent_heap_used,
    threads_blocked,
    rpc_callqueue_len,
    tt_last_seen,
    mr_failure_count,
    mr_running_maps,
    mr_running_reduces,
    dn_last_seen,
    dn_disk_space,
    dn_avg_blockreport_time,
    dn_block_verification_failures]


def get_all_extractors():
  return [e for e in globals().values()
          if isinstance(e, Extractor)]
