#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
#
# Ganglia is an open source monitoring framework that is
# very widely installed. This Cluster implementation connects to a ganglia
# meta daemon which aggregates Ganglia info from many nodes
# and emits the supplied metrics as DataPoints.
#
# Note: Ganglia 3.0 and 3.1 should both be supported.
#
# TODO(henry): There is more Ganglia detail that we can capture.
# For example, Ganglia supports the notion of clusters, which maybe
# we want to do something with.
#
# TODO(henry): Ganglia appears 'down' if there's a version incompatibility.
# I'd prefer a more meaningful message.

# TODO(henry): Replace dictionaries returned by GangliaProxy with a more
# readable object model.

from health.models import GMETAD, GANGLIA
from health.clusters import Cluster
from health.data import DataPoint
import health.conf

from ganglia_util import GangliaProxy

import logging
import socket

# TODO(philip): Each of the 3 clusters that we support right now
# has one of these caches somewhere, where data is extracted from
# the configuration, and placed into a map.  We should generalize
# this approach.
GANGLIA_CACHE = None

def all_ganglias():
  global GANGLIA_CACHE
  if GANGLIA_CACHE is not None:
    return GANGLIA_CACHE
  GANGLIA_CACHE = {}

  for name in health.conf.GANGLIA_CLUSTERS.keys():
    conf = health.conf.GANGLIA_CLUSTERS[name]
    try:
      GANGLIA_CACHE[name] = GangliaCluster(
        name,
        conf.GMETAD_HOST.get(), 
        conf.GMETAD_PORT.get(), 
        conf.GANGLIA_URL.get())
    except:
      logging.exception("Failed to initialize Ganglia cluster: %s" % name)
      pass
  return GANGLIA_CACHE

class GangliaCluster(Cluster):
  """
  Emits Ganglia info queried direct from gmetad.

  Each data point is keyed by the metric name (e.g. load). It's value is
  a dictionary of attributes for that sample; including units, long description
  and so on (including a URL that's put together here pointing to a graph for the
  historical data).

  Note that we're not monitoring Ganglia here, but maybe we should via particpants;
  otherwise I'm concerned that GMETAD failing will give false negatives for the
  nodes it monitors.
  """
  def __init__(self, name, host, port, url):
    """
    Only thing we do here is to create a proxy object which
    talks directly to Ganglia's GMETAD. I've kept this separate
    in case we ever want to talk to several instances of GMETAD (which
    itself aggregates nodes); however for this version we only have one
    proxy (and in fact, they're essentially stateless... so can probably
    just keep the one around).
    """
    super(GangliaCluster, self).__init__(GANGLIA, name)
    self.url = url
    self.proxy = GangliaProxy(host, port, url + "graph.php")

  def get_fresh_datapoints(self):
    """
    Will always return at least one datapoint per participant
    """
    def to_dict(ip):
      """
      Make a participant-compatible dictionary from its ip. 
      """
      return dict(id = ip,
                  ip=socket.gethostbyname(self.proxy.host),
                  http_port=self.proxy.port, name=ip)

    # Metrics is a dictionary which maps IPs onto a
    # dictionary from metric names onto yet another dictionary
    # of attributes. i.e. metrics["192.168.88.156"]["CPU"]["VAL"]
    try:
      metrics = self.proxy.poll_gmetad()
    except Exception, e:
      # TODO(philip): This doesn't report that Ganglia is down
      # when there's an exception.  Need to separate out GMOND and GMETAD,
      # and follow same logic as NN and JT does.
      logging.exception(e)
      metrics = {}
    
    from clusters import coalesce_participants
    for participant, report in coalesce_participants(
      self,
      GMETAD,
      [ to_dict(k) for k in metrics ], # Participants are identified by IP
      [ metrics[k] for k in metrics]): # Join data is the dictionary of metrics
      if report is not None:
        yield DataPoint(self, participant, *DataPoint.STATE_UP)
        for datum in report: 
          yield DataPoint(self, participant, datum,report[datum])
      else:
        yield DataPoint(self, participant, *DataPoint.STATE_DOWN)
