#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
#
# Utilities for Ganglia monitoring.
#

import xml.parsers.expat
import socket
import logging
from urllib import quote_plus

class GangliaProxy(object):
  """
  We use expat to quick-and-dirtily parse the XML that GMETAD gives us.
  Rather than maintain a single connection, we have to keep connecting since
  GMETAD shuts the socket once we've done a read.
  """
  def __init__(self,host,port,url=""):
    self.host, self.port = host, port
    # Following variables capture the relevant state of the parser
    self.in_cluster = None
    self.in_host = None
    self.metricname = None
    self.url = url

  def poll_gmetad(self):
    """
    This is the entry point from GangliaManager.
    """
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.connect((self.host,self.port))
    try:
      self.parse(sock.makefile("r"))
    except Exception, e:
      # I'm not sure I care why this failed (failure of parse or makefile)
      logging.exception("Failed to parse data from Ganglia!")
      raise
    sock.close()
    if self.good_version:
      return self.values
    return {}

  def parse(self, file):
    self.values, self.good_version = {}, True
    parser = xml.parsers.expat.ParserCreate()
    parser.StartElementHandler = self.start_element
    parser.EndElementHandler = self.end_element
    parser.ParseFile(file)

  def start_element(self, name, attrs):
    if name == "CLUSTER":
      self.in_cluster = attrs["NAME"]
    elif name == "HOST":
      self.in_host = attrs["NAME"]
    elif self.in_host and name == "METRIC":
      self.values.setdefault(self.in_host,{})[attrs["NAME"]] = attrs
      self.metricname = attrs["NAME"]
    elif name == "EXTRA_ELEMENT":
      # This element is specific to Ganglia 3.1.x
      self.values[self.in_host][self.metricname][attrs["NAME"]] = attrs["VAL"]
    elif name == "GANGLIA_XML":
      if "VERSION" not in attrs:
        logging.warn("Unable to find Ganglia version; continuing but compatibility problems may ensue.")
      elif not (attrs["VERSION"].startswith("3.1.") or attrs["VERSION"].startswith("3.0.")):
        raise Exception("Incompatible Ganglia version %s (we support 3.0.x and 3.1.x)" % attrs["VERSION"])

  def end_element(self, name):
    if name == "HOST" and self.in_host:
      self.in_host = None
    elif name == "CLUSTER" and self.in_cluster:
      self.in_cluster = None
    elif name == "METRIC":
      # Generate the URL now that we've (hopefully) got the
      # extra info to do so
      attrs = self.values[self.in_host][self.metricname]
      if attrs is not None:
        subst = {'url': self.url,
                 'cluster': quote_plus(self.in_cluster),
                 'host': quote_plus(self.in_host),
                 'name': quote_plus(attrs["NAME"]),
                 # Both UNITS and TITLE are optional in both 3.0 and 3.1, for different reasons
                 'units': quote_plus(attrs.get("UNITS","")),
                 'title': quote_plus(attrs.get("TITLE","")),
                 'value': quote_plus(attrs["VAL"])}
        attrs["URL"] = "%(url)s?&c=%(cluster)s&h=%(host)s&m=%(name)s&r=hour&jr=&js=&vl=%(units)s&ti=%(title)s&v=%(value)s" % subst
