#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
"""
Twisted-Based daemon that runs alongside Desktop to periodically
poll all of the daemons in the cluster for metrics, and potentially
other health information in the future.

This daemon should be started by running bin/desktop run_healthd
It exposes its data at the simple URL http://<host>:8080/?format=<fmt>
Format can be 'json' (default) or 'pickle' which is faster when
dealing with a python client.
"""

import cPickle
import logging
import random
import simplejson
from twisted.internet import reactor, defer, threads
from twisted.web import client, resource, server

from health.models import HDFS, MR, Participant

from health import conf
import desktop.conf

LOG = logging.getLogger("healthd")

class MetricsStore(object):
  """Storage layer for holding all of the current metrics data in RAM.
  Currently just a thin layer around a dictionary, but we may want to
  move this to sqlite in the future, or hang loggers off for historical
  data tracking, etc.
  """
  def __init__(self):
    self.metrics = {}

  def store_metrics(self, hostport, data):
    self.metrics[hostport] = data

  def store_error(self, hostport):
    if hostport in self.metrics:
      del self.metrics[hostport]

  def get_all_metrics(self):
    return self.metrics


class FetcherManager(object):
  """
  Manages the MetricsFetcher instances by periodically refreshing
  its view of the cluster node membership by means of polling any number of
  WebUiSources.
  """

  # How long between fetching up to date node lists
  REFRESH_INTERVAL = conf.HEALTHD.MEMBERSHIP_REFRESH_INTERVAL.get()

  def __init__(self, store, webui_source=None):
    """
    @param store: a MetricsStore that data will fetch into
    @param webui_source: a WebUiSource used to determine membership (defaults to ParticipantDbSource)
    """
    self.fetchers = {} # map from participant id -> MetricsFetcher instance
    self.store = store
    self.webui_source = webui_source or ParticipantDbSource()

  @defer.deferredGenerator
  def trigger_refresh(self):
    """
    Triggers a refresh of node membership and automatically schedules
    itself to run again after REFRESH_INTERVAL.
    """
    LOG.info("Refreshing node information from master nodes")
    d = defer.waitForDeferred(self._do_refresh())
    yield d
    try:
      d.getResult()
    finally:
      # Schedule next refresh regardless of whether there was an error
      reactor.callLater(self.REFRESH_INTERVAL, self.trigger_refresh)

  @defer.deferredGenerator
  def _do_refresh(self):
    web_uis = {}
    # Grab the list of web UIs from our source
    uis = defer.waitForDeferred(
      defer.maybeDeferred(self.webui_source.get_webuis))
    yield uis
    uis = uis.getResult()
    web_uis.update(uis)

    # Delete fetchers that are running but shouldn't be
    for k, fetcher in self.fetchers.iteritems():
      if k not in web_uis:
        LOG.info("Removing metrics endpoint: " + k)
        fetcher.stop()
        del self.fetchers[k]

    # Add fetchers that aren't running but should be
    for p_id, web_ui in web_uis.iteritems():
      if p_id not in self.fetchers:
        LOG.info("Adding metrics endpoint: " + web_ui)

        fetcher = MetricsFetcher(p_id, web_ui, self.store)
        delay = random.random() * MetricsFetcher.FETCH_INTERVAL
        reactor.callLater(delay, fetcher.trigger_fetch)
        self.fetchers[p_id] = fetcher



class WebUiSource(object):
  """Interface for a source for cluster membership information."""

  def get_webuis(self):
    """
    Implementations should return a (maybe-deferred) dictionary of
    participant id -> host:port, each representing a Hadoop Web UI.
    """
    return {}


class ParticipantDbSource(object):
  """
  Web UI Source which uses the django Participant model to discover all node
  members that expose Hadoop Web UIs.
  """
  @defer.deferredGenerator
  def get_webuis(self):
    try:
      desktop_url = "http://%s:%d/health/update_state" % (
            desktop.conf.HTTP_HOST.get(), desktop.conf.HTTP_PORT.get())
      page = defer.waitForDeferred(client.getPage(desktop_url,
                                                  timeout=10))
      yield page
      page = page.getResult()
      assert page.strip() == "OK"
    except:
      logging.exception("Couldn't ping desktop at " + desktop_url + 
                        " to force state refresh. " +
                        "Using potentially stale data.")

    parts = defer.waitForDeferred(
      threads.deferToThread(Participant.objects.all))
    yield parts
    parts = parts.getResult()

    uis = {}
    for p in parts:
      if p.cluster.type in [HDFS, MR] and p.http_port:
        # cast to str because these are unicode in db and getPage fails with
        # unicode URLs
        uis[p.id] = str("%s:%d" % (p.ip, p.http_port))
    yield uis

class MetricsFetcher(object):
  """
  Responsible for periodically fetching metrics from a web ui and populating
  them into a MetricsStore
  """

  # How long to wait for a response from a hadoop server
  FETCH_TIMEOUT = conf.HEALTHD.METRICS_FETCH_TIMEOUT.get()

  # How long in between fetches for a single server
  FETCH_INTERVAL = conf.HEALTHD.METRICS_FETCH_INTERVAL.get()

  def __init__(self, participant_id, hostport, store):
    """
    @param participant_id: the participant ID we're fetching for
    @param hostport: the host:port for the web UI
    @param store: the MetricsStore to populate data into
    """
    self.participant_id = participant_id
    self.hostport = hostport
    self.store = store
    self.timeoutCall = None
    self.running = True

  def trigger_fetch(self):
    url = "http://%s/metrics?format=json" % self.hostport
    client.getPage(url, timeout=self.FETCH_TIMEOUT).addCallbacks(self.success, self.error)

  def stop(self):
    if self.timeoutCall:
      self.timeoutCall.cancel()
    # We may be mid-fetch, so no call is scheduled, but we still need to keep
    # the success/error callbacks from running
    self.running = False

  def success(self, page):
    if not self.running: return
    data = simplejson.loads(page)
    self.store.store_metrics(self.participant_id, data)
    LOG.info("Fetched metrics from %s" % self.hostport)
    self.scheduleNext()

  def scheduleNext(self):
    self.timeoutCall = reactor.callLater(
      self.FETCH_INTERVAL, self.trigger_fetch)

  def error(self, error):
    if not self.running: return
    print "Error: ", error
    self.store.store_error(self.participant_id)
    self.scheduleNext()
    return error

class MetricsDumpResource(resource.Resource):
  """
  A Twisted Web resource that dumps all of the metrics in the current MetricsStore.
  """

  isLeaf = True
  def __init__(self, store):
    resource.Resource.__init__(self)
    self.store = store

  def render_GET(self, request):
    """Dump the metrics store to the user"""
    format = request.args.get("format", ["json"])[0]
    if format == "json":
      request.setHeader('content-type', 'text/javascript')
      pretty = request.args.get("pretty", ["0"])[0] in ['1', 'true']
      return simplejson.dumps(self.store.get_all_metrics(),
                              indent=(pretty and 2))
    elif format == "pickle":
      request.setHeader('content-type', 'application/x-python-pickle')
      return cPickle.dumps(self.store.get_all_metrics())
    else:
      raise Exception("Unknown format!")

def main():
  logging.basicConfig(level=logging.INFO)

  store = MetricsStore()
  fetcher_mgr = FetcherManager(store)
  fetcher_mgr.trigger_refresh()

  site = server.Site(MetricsDumpResource(store))
  port = conf.HEALTHD.HTTP_PORT.get()
  reactor.listenTCP(port, site)
  LOG.info("Listening on port %d" % port)
  reactor.run()

if __name__ == "__main__":
  main()
