#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
"""
Library of HeatDetectors for displaying measurements in a Report in
heat-map form.
"""

class HeatDetector(object):
  """
  A Heat Detector is responsible for determining an intensity of a cell
  in a report. It should return a float between 0 and 1, where 1.0 indicates
  "hot" (bad/red) and 0.0 indicates "normal" (good/green)

  These measurements can (and often should) be made relative to the other
  cells of that extractor in the given report, since normal values for one
  cluster may be abnormal for another.
  """
  def get_intensity_for_cell(self, cell, report):
    pass

class HighBadDetector(HeatDetector):
  """
  Heat detector that returns high intensity when the cell's value
  is a high outlier.
  """
  def get_intensity_for_cell(self, cell, report):
    stats = report.get_summary_statistics(cell.extractor)
    above = cell.value - stats['q3']
    above_frac = min(max(float(above) / (stats['iqr']*1.5), 0.0), 1.0)
    return above_frac

class LowBadDetector(HeatDetector):    
  """
  Heat detector that returns high intensity when the cell's value is a
  low outlier.
  """
  def get_intensity_for_cell(self, cell, report):
    stats = report.get_summary_statistics(cell.extractor)
    below = stats['q1'] - cell.value
    below_frac = min(max(float(below) / (stats['iqr']*1.5), 0.0), 1.0)
    return below_frac

class AbnormalBadDetector(HeatDetector):
  """
  Heat detector that returns high intensity when the cell's value is an
  outlier in either direction.
  """
  def get_intensity_for_cell(self, cell, report):
    stats = report.get_summary_statistics(cell.extractor)
    # Intensity is how far outside middle 50%
    outside = max(cell.value - stats['q3'], stats['q1'] - cell.value, 0.0)
    return outside/(2*stats['iqr'])

class ThresholdDetector(HeatDetector):
  def __init__(self, low_worst, low_start, high_start, high_worst):
    self.low_worst = low_worst
    self.low_start = low_start
    self.high_start = high_start
    self.high_worst = high_worst

  def get_intensity_for_cell(self, cell, report):
    if self.low_start is not None and cell.value < self.low_start:
      below = self.low_start - cell.value
      pct = float(below) / (self.low_start - self.low_worst)
      return min(pct, 1.0)
    elif self.high_start is not None and cell.value > self.high_start:
      above = cell.value - self.high_start
      pct = float(above) / (self.high_worst - self.high_start)
      return min(pct, 1.0)
    else:
      # within range
      return 0
