#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
"""
Framework for lint checks across health data.
"""
import logging
import operator
from desktop.lib.view_util import big_filesizeformat
from health.models import Participant, NAME_NODE, JOB_TRACKER, GMETAD
from health.clusters import Cluster
from health.conf import LINT_OVERRIDES

LOG = logging.getLogger(__name__)

# Lint levels
CRITICAL = 'CRITICAL'
WARN = 'WARN'
NOTICE = 'NOTICE'
OK = 'OK'

# All LintRoller subclasses that have been imported magically
# register themselves here via LintRoller.__metaclass__
ROLLER_REGISTRY = []

class LintRoller(object):
  """A LintRoller is responsible for investigating the full set of health
  DataPoints and outputting some number of LintCheck instances.

  Any LintRoller subclasses that are defined will be automatically registered
  and picked up by the lint system.
  """

  class __metaclass__(type):
    """Metaclass that registers all known LintRollers in ROLLER_REGISTRY"""
    def __init__(cls, name, bases, class_dict):
      type.__init__(cls, name, bases, class_dict)

      # We dont want to register LintRoller itself as an implementation
      if len(bases) == 1 and bases[0] == object:
        return
      # Allow classes to opt out (for tests, for example)
      if class_dict.get('skip_registration'):
        return
      ROLLER_REGISTRY.append(cls)

  COMPARATORS = {
      '<':  operator.lt,
      '<=': operator.le,
      '>':  operator.gt,
      '>=': operator.ge,
      }
  COMPARATOR_NEGATIONS = {
    '<': '>=',
    '<=': '>',
    '>': '<=',
    '>=': '<'}

  def run_checks(self, datapoints):
    """
    Implementations should iterate over the datapoints and yield any number of
    LintCheck instances.
    """
    pass


class LintCheck(object):
  """
  A LintCheck is the output of a particular check against the state of the world.
  Each time a LintRoller runs a check against some part of the cluster, it should yield
  a LintCheck instance describing the result of that check.

  Instances must define the following class members:
    - id   - a unique text identifier for this check
    - name - a human readable (but short) name for this check
    - good_message - a message that will be displayed for 'OK' results
    - bad_message - a message that will be displayed for non-'OK' results

  The bad_message should endeavour to provide some form of advice to the
  user for how to rectify the issue.
  """

  class __metaclass__(type):
    """Metaclass that allows all class attributes to be overridden by configuration
    parameters. For example, the user can write

    [foo_lint_check]
    help="Blah blah specific help for this cluster"
    warn_level = 234234
    """
    def __init__(cls, name, bases, class_dict):
      type.__init__(cls, name, bases, class_dict)
      if "id" in class_dict:
        # Perform overrides
        overrides = LINT_OVERRIDES.get().get(class_dict['id'])
        if overrides:
          for k, v in overrides.get_data_dict().iteritems():
            t = type(getattr(cls, k))
            setattr(cls, k, t(v))


  def __init__(self, subject, status, detail=None):
    """
    @param subject: the item that has been checked (Cluster or Participant)
    @param status: one of the constants at the top of this module
    @param detail: A string detail message regarding the result of the check
    """
    self.subject = subject
    self.status = status
    self.detail = detail
    if self.status == OK:
      self.message = self.good_message
    else:
      self.message = self.bad_message

  def to_jsonable(self):
    return dict(id=self.id,
                name=self.name,
                subject=self.subject,
                message=self.message,
                status=self.status,
                detail=self.detail)


class ThresholdLintCheck(LintCheck):
  """A lint check that is threshold-based. Subclasses should set the
  class members 'comparison', 'warn_level' and 'critical_level'.

  @var comparison: one of the keys of LintRoller.COMPARATORS
  @var warn_level: if value doesn't compare to warn_level, raises a WARN level lint
  @var critical_level: if value doesn't compare to warn_level, raises a CRITICAL level lint
  """

  @classmethod
  def check(cls, subject, value):
    """
    Check the comparison of a value against given warning and critical thresholds.

    e.g.

    class MyLintCheck(ThresholdLintCheck):
      ... [standard LintCheck attributes]
      comparison = '>='
      warn_level = 20
      critical_level = 10

    MyLintCheck.check_threshold(some_cluster, percent_free)
      will raise a WARN if percent_free is not >= 20%
      will raise a CRITICAL if percent_free is not >= 10%

    @param subject: the cluster or participant for which this lint should be reported
    @param value: the value to check (comparable)
    """
    comparator = LintRoller.COMPARATORS[cls.comparison]

    if not comparator(cls.warn_level, cls.critical_level):
      raise Exception("The critical level is reached before the warn level")

    if not comparator(value, cls.critical_level):
      return cls(
        subject=subject,
        status=CRITICAL,
        detail="Value %s %s critical threshold %s" % (
          cls.format_value(value),
          LintRoller.COMPARATOR_NEGATIONS[cls.comparison],
          cls.format_value(cls.critical_level)))
    elif not comparator(value, cls.warn_level):
      return cls(
        subject=subject,
        status=WARN,
        detail="Value %s %s warning threshold %s" % (
          cls.format_value(value),
          LintRoller.COMPARATOR_NEGATIONS[cls.comparison],
          cls.format_value(cls.warn_level)))
    else:
      return cls(
        subject=subject,
        status=OK,
        detail="Value %s %s %s" % (
          cls.format_value(value),
          cls.comparison,
          cls.format_value(cls.warn_level)))


  @classmethod
  def format_value(cls, val):
    """Formats the given threshold or value for inclusion in a detail statement.

    Subclasses may want to pick an alternative implementation, e.g. by:
      format_value = ThresholdLintCheck.format_value_pct
    """
    return str(val)

  @classmethod
  def format_value_size(cls, val):
    return big_filesizeformat(val)

  @classmethod
  def format_value_percent(cls, val):
    return "%2d%%" % (val * 100)

def run_all_lints(data):
  from health import lint_checks
  return run_lint_rollers(data, ROLLER_REGISTRY)

def run_lint_rollers(data, roller_classes):
  """
  Runs the given lint rollers across the health datapoints.

  @param data: a list of DataPoints
  @param roller_classes: list of classes that inherit from LintRoller
  """
  lints = []
  for roller_class in roller_classes:
    roller = roller_class()
    for lint in roller.run_checks(data):
      lints.append(lint)
  return lints


def sort_lints(lints):
  """
  Sort lints by descending importance. The order of importance is defined as:
    - Cluster CRITICAL, WARN, NOTICE in that order
    - Machine CRITICAL, WARN, NOTICE in that order
    - For machines, master machines are prioritized first
    - If all of the others are the same, sort by machine name then lint check

  TODO(todd) testme
  """
  def index_or_negone(l, val):
    """Like list.index but returns -1 for not found"""
    try:
      return l.index(val)
    except ValueError:
      return -1

  def key_func(lint):
    # Orders from least important to most
    level_order = [OK, NOTICE, WARN, CRITICAL]
    class_order = [Participant, Cluster]
    role_order = [GMETAD, JOB_TRACKER, NAME_NODE]

    level_idx = index_or_negone(level_order, lint.status)
    class_idx = index_or_negone(class_order, lint.subject.__class__)
    role_idx = -1

    if isinstance(lint.subject, Participant):
      role_idx = index_or_negone(role_order, lint.subject.role)


    return (level_idx, class_idx, role_idx, lint.subject.name, lint.name)
  lints.sort(key=key_func, reverse=True)
