#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
"""Definitions of the basic hadoop lint checks"""

import logging

from health.lint import ThresholdLintCheck, LintRoller

LOG = logging.getLogger(__name__)

class FreeSpaceCheck(ThresholdLintCheck):
  id = "dfs_free_space"
  name = "DFS Free Space"
  good_message = "There is sufficient free space on this DFS cluster."
  bad_message = "Insufficient free space is available on this cluster."
  comparison = '>='
  critical_level = 100*1024*1024*1024 # 100G
  warn_level = 200*1024*1024*1024 # 200G
  format_value = ThresholdLintCheck.format_value_size

class DeadDataNodesCheck(ThresholdLintCheck):
  id = "dead_datanodes"
  name = "Dead Datanodes"
  good_message = "There are no dead datanodes on this cluster."
  bad_message = "Some datanodes have gone down on this cluster."
  comparison = '<='
  warn_level = 1
  critical_level = 3

class DfsHealthReportRoller(LintRoller):
  """
  Lint roller that runs through the DFS Health Reports for each
  HDFS cluster.
  """
  def run_checks(self, data):
    # Aggregate all of the health report information in a dict
    # cluster -> dict(health report key -> value)
    health_reports = dict()
    for dp in data:
      if (dp.cluster and dp.cluster.type == 'HDFS' and
          dp.participant is None):
        if dp.key.startswith("dfs_health_report"):
          _, key = dp.key.split(':', 1)
          health_reports.setdefault(dp.cluster, {})[key] = dp.value

    for cluster, health in health_reports.iteritems():
      LOG.debug("cluster %s health: %s" % (cluster, repr(health)))
      yield FreeSpaceCheck.check(cluster, health['bytesRemaining'])
      yield DeadDataNodesCheck.check(cluster, health['numDeadDataNodes'])


class DataNodeFreeSpaceCheck(ThresholdLintCheck):
  id = "dn_free_space"
  name = "Datanode Free Space"
  good_message = "There is sufficient free space on this datanode."
  bad_message = "Insufficient free space is available on this datanode."
  comparison = '>='
  warn_level = 0.15
  critical_level = 0.05
  format_value = ThresholdLintCheck.format_value_percent

class NeedsBalancerCheck(ThresholdLintCheck):
  id = "needs_balancer"
  name = "Data Balancing"
  good_message = "The data is evenly distributed in your cluster."
  bad_message = "The data in your cluster is not evenly distributed. " + \
      "Consider running the DFS balancer."
  # The number that gets fed in here is max(abs(datanode_pct_used - cluster_pct_used)))
  # That is to say, if the cluster is 30% full and the most out-of-balance datanode is
  # 12% full, the input value will be 18% (0.18)
  comparison = '<='
  warn_level = 0.20
  critical_level = 1.0
  format_value = ThresholdLintCheck.format_value_percent

class DataNodeReportRoller(LintRoller):
  """
  Lint roller that runs through each datanode's report.
  """
  def run_checks(self, data):
    # Aggregate datanode reports by datanode
    dn_reports = dict()
    for dp in data:
      if (dp.cluster and dp.cluster.type == 'HDFS' and
          dp.participant is not None and
          dp.key.startswith("datanode_report")):
        _, key = dp.key.split(':', 1)
        dn_reports.setdefault(dp.participant, {})[key] = dp.value

    # Check space remaining on all of the datanodes
    # Also, aggregate total capacity and used space on the cluster
    # so we can check if it's balanced. This mirrors the code in
    # Balancer.java
    default = {
        'num_nodes': 0,
        'total_used': 0,
        'total_cap': 0,
        'worst_outofbalance': 0,
    }
    cluster_totals = dict()

    for dn, report in dn_reports.iteritems():
      cluster_totals[dn.cluster] = default.copy()

      if report['capacity'] > 0:
        pct_remaining = float(report['remaining'])/report['capacity']
      else: # avoid div by zero
        pct_remaining = 0
      yield DataNodeFreeSpaceCheck.check(dn, pct_remaining)

      cluster_totals[dn.cluster]['num_nodes'] += 1
      cluster_totals[dn.cluster]['total_cap'] += report['capacity']
      cluster_totals[dn.cluster]['total_used'] += report['dfsUsed']

    # Check that the cluster is balanced
    for dn, report in dn_reports.iteritems():
      # Skip empty DNs from balancing checks
      if report['capacity'] == 0 or cluster_totals[dn.cluster]['total_cap'] == 0:
        continue

      # Percent used for this DN
      pct_used = float(report['dfsUsed'])/report['capacity']

      # Percent used for this DN's cluster
      cluster_used = (float(cluster_totals[dn.cluster]['total_used']) /
                      cluster_totals[dn.cluster]['total_cap'])
      cluster_totals[dn.cluster]['worst_outofbalance'] = max(
        cluster_totals[dn.cluster]['worst_outofbalance'],
        abs(pct_used - cluster_used))

    # Yield the lints for balancing
    for cluster, totals in cluster_totals.iteritems():
      yield NeedsBalancerCheck.check(cluster, totals['worst_outofbalance'])

class JvmHeapCheck(ThresholdLintCheck):
  id = "jvm_heap"
  name = "JVM Heap Usage"
  good_message = "There is sufficient free heap space on this daemon."
  bad_message = "Insufficient free heap space is available on this daemon."
  comparison = '<='
  warn_level = 0.75
  critical_level = 0.90
  format_value = ThresholdLintCheck.format_value_percent

class RuntimeInfoRoller(LintRoller):
  """
  Lint roller that looks for nodes that are close to exhausting heap space.
  """
  def run_checks(self, data):
    by_participant = {}
    for dp in data:
      if dp.key.startswith("runtime_info"):
        _, key = dp.key.split(':', 1)
        by_participant.setdefault(dp.participant, dict())[key] = dp.value

    for participant, runtime_info in by_participant.iteritems():
      yield JvmHeapCheck.check(
        participant, float(runtime_info['totalMemory'])/runtime_info['maxMemory'])
