#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.

from django.core import urlresolvers
from django.db import models
import re

# Django doesn't support good enums; we use strings for simplicity.
# Cluster types.
CLUSTER_TYPES = HDFS, MR, GANGLIA = "HDFS", "MR", "GANGLIA"

ROLE_TYPES \
  = (NAME_NODE, DATA_NODE, SECONDARY_NAME_NODE,
     JOB_TRACKER, TASK_TRACKER, GMETAD) \
  = ("NAME_NODE", "DATA_NODE", "SECONDARY_NAME_NODE",
     "JOB_TRACKER", "TASK_TRACKER", "GMETAD") \

MAX_TYPE_LEN=32

assert MAX_TYPE_LEN >= max([ len(x) for x in CLUSTER_TYPES ])
assert MAX_TYPE_LEN >= max([ len(x) for x in ROLE_TYPES ])

def pair(x):
  return x,x

def make_choices(x):
  map(pair, x)

class Participant(models.Model):
  """
  Participants belong to a specific cluster (see clusters.py), 
  and represent, typically, a daemon running on a host.

  Clusters are read from the configuration file,
  rather than the database, so instead of a foreign
  key, we use a (type, name) pair.
  """
  class Meta:
    # A composite key.
    unique_together = ( ("id", "role", "cluster_type", "cluster_name") )

  # The primary key for a participant may be cluster
  # specific, but is typically ip:port.  Django
  # doesn't support composite primary keys, so
  # this must be managed manually.
  id = models.CharField(max_length=100, primary_key=True)
  cluster_type = models.CharField(
    choices= (make_choices(CLUSTER_TYPES)),
    max_length = MAX_TYPE_LEN,
  )
  cluster_name = models.CharField(max_length=128)
  role = models.CharField(
    choices = (make_choices(ROLE_TYPES)),
    max_length = MAX_TYPE_LEN,
  )
  ip = models.IPAddressField()
  name = models.CharField(max_length=128)

  # Port where our plugin thrift service is running
  thrift_port = models.IntegerField(null=True)

  # Port where the Hadoop Web UI is running
  http_port = models.IntegerField(null=True) # nullable when unknown

  @property
  def cluster(self):
    from health.clusters import Cluster
    return Cluster.get_cluster(self.cluster_type, self.cluster_name)

  @property
  def role_nice(self):
    def repl(match):
      return match.groups()[0].upper()
    return re.sub(r'\b(\w)', repl, self.role.lower().replace("_", " "))

  @property
  def role_abv(self):
    roles = {
      DATA_NODE: "dn",
      JOB_TRACKER: "jt",
      NAME_NODE: "nn",
      SECONDARY_NAME_NODE: "snn",
      TASK_TRACKER: "tt",
      GMETAD: "gmeta",
    };
    return roles.get(self.role, self.role).upper()

  @property
  def short_name(self):
    if self.http_port:
      http_port = str(self.http_port)
    else:
      http_port = "?"
    return "%s @ %s:%s" % (self.role_abv, self.ip, http_port)

  @property
  def http_proxy_url(self):
    """
    Return a proxy URL for this health participant.
    If the participant's HTTP port is unknown (eg because it's down)
    this may return None, signifying that the participant
    is inaccessible.
    """
    if self.http_port is None:
      return None

    url = urlresolvers.reverse("proxy.views.proxy", 
      kwargs=dict(host=self.ip, port=self.http_port, path="/"))
    return url

  def __str__(self):
    return (("Participant(%s, cluster_type=%s, cluster_name=%s, role=%s, " +
             "ip=%s, thrift_port=%s, http_port=%s, name=%s)") %
            (self.id, self.cluster_type, self.cluster_name, self.role,
             self.ip, self.thrift_port, str(self.http_port), self.name))
