# Copyright (c) 2010, Cloudera, inc. All rights reserved.

def group(dataset, grouping):
  d = dict()
  for x in dataset:
    g = grouping(x)
    if g is not None:
      d.setdefault(g, list()).append(x)
  return d


def query(data, select, groupby=None, where=None, having=None):
  """
  Emulates a SQL-style group by and aggregation query.

  If there is a groupby:
    data is an iterable of data rows.
    select grouping, values -> output is a function applied to (grouping, values)
    where: row->bool restricts the original data rows
    groupby: row->grouping groups the data
    having grouping, values -> bool restricts the output
  If there's no groupby, select is applied to every row
  where the where function is true.
  """
  # The no-grouping case is very simple:
  if groupby is None:
    assert having is None
    return map(select, filter(where, data))

  grouped = {}
  for row in data:
    if where is None or where(row):
      grouping = groupby(row)
      grouped.setdefault(grouping, []).append(row)

  output = []
  for grouping, values in grouped.items():
    if having is None or having(grouping, values):
      selection = select(grouping, values)
      output.append(selection)
  return output

def find_single(data, select, where, default):
  """
  Returns a single row matching the where clause.
  Asserts that only one row matches.
  """
  filtered = filter(where, data)
  assert len(filtered) <= 1
  if filtered:
    return select(filtered[0])
  else:
    return default

def remap(data, key_mapping):
  """
  For datapoints in data where keys are in key_mapping,
  returns a key-value map with keys in key_mapping.
  """
  d = {}
  for dp in data:
    key = key_mapping.get(dp.key)
    if key is not None:
      assert key not in d
      d[key] = dp.value
  return d

def average(data):
  """Average function that returns 0 if there's no data."""
  if not data:
    return 0
  return sum(data)/len(data)

def bucket_count(datapoints, key, buckets):
  """
  Counts occurrences of values in buckets in the 
  datapoints with associated key.
  """
  d = {}
  for b in buckets:
    d[b] = 0
  for dp in datapoints:
    if dp.key == key:
      d[dp.value] += 1
  return d
