#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.

from health import stats
import math

class Report(object):
  """
  A Report is generated by running a set of extractors on a set of datapoints.
  The extractors can be thought of columns. The rows are participants.

  The report is sparse in that not every extractor applies to every participant.

  Access can be done either row-major or column-major by using either the
  cells_by_participant or cells_by_extractor members. cells_by_participant
  is a mapping participant -> extractor -> cell. cells_by_extractor is a mapping
  extractor -> participant -> cell.
  """
  def __init__(self, datapoints, extractors):
    self.extractors = extractors
    self.cells_by_participant, self.cells_by_extractor = self._generate_cells(datapoints)
    self.memoize_stats = dict()

  def _generate_cells(self, datapoints):
    cells = []
    cells_by_extractor = dict()
    cells_by_participant = dict()

    dp_by_participant = dict()
    for dp in datapoints:
      if not dp.participant: continue
      dp_by_participant.setdefault(dp.participant, []).append(dp)

    for participant, data in dp_by_participant.iteritems():
      cells_by_participant.setdefault(participant, {})
      data_dict = dict( [(dp.key, dp.value) for dp in data] )
      for extractor in self.extractors:
        val = extractor.extract(data_dict)
        if val is not None:
          cell = ReportCell(participant, extractor, val)
          cells_by_participant[participant][extractor] = cell
          cells_by_extractor.setdefault(extractor, {})[participant] = cell
    return cells_by_participant, cells_by_extractor

  def to_jsonable(self):
    ret = dict()
    for participant, data in self.cells_by_participant.iteritems():
      p_dict = dict()
      for extractor, cell in data.iteritems():
        p_dict[extractor.name] = cell.value
      ret[participant.name] = p_dict

    return ret

  def get_intensity_for_cell(self, cell):
    det = cell.extractor.heat_detector
    if det:
      return det.get_intensity_for_cell(cell, self)
    return None

  def get_summary_statistics(self, extractor):
    """
    Return a dictionary of the summary statistics for a given extractor
    across all participants. This dictionary has keys:
      - 'iqr': the interquartile range of the sample
      - 'q1': the first quartile (25th percentile)
      - 'q3': the third quartile (75th percentile)
    The results of this function are memoized for a given report.
    """
    if extractor in self.memoize_stats:
      return self.memoize_stats[extractor]

    vals = [mycell.value for mycell in self.cells_by_extractor[extractor].itervalues()]
    q1, q3 = stats.quantile(vals, 0.25), stats.quantile(vals, 0.75)

    # If we started with integers, let's round the quartile measurements to the
    # outside integer - this helps out when metrics have values like 2,2,3,5 - we
    # would rather have our quartiles be values that are actually possible, even
    # though statistically it's not particularly justified.
    if isinstance(vals[0], int):
      q1 = math.floor(q1)
      q3 = math.ceil(q3)
    ret = dict(q1=q1,
               q3=q3,
               iqr=max(q3 - q1, 0.00001))
    self.memoize_stats[extractor] = ret
    return ret

class ReportCell(object):
  """
  A cell in a report is simply a participant (row), extractor (column),
  and the extracted value.
  """
  def __init__(self, participant, extractor, value):
    self.participant = participant
    self.extractor = extractor
    self.value = value

  @property
  def formatted_value(self):
    return self.extractor.formatter(self.value)
