#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
"""
Standard report types for different Hadoop daemons.

A report groups a bunch of extractors together, so the classes
in this module define groups of extractors that we know to apply
to specific node types.
"""
from health.report import Report
from health import extractors
from health.models import DATA_NODE, TASK_TRACKER, NAME_NODE, JOB_TRACKER


class StandardReport(Report):
  def __init__(self, datapoints):
    super(StandardReport, self).__init__(datapoints, self.extractors)

class JvmReport(StandardReport):
  name = "JVM Metrics"
  extractors = [
    extractors.threads_blocked,
    extractors.jvm_heap_max,
    extractors.jvm_heap_used,
    extractors.jvm_percent_heap_used,
    ]


class TaskTrackerReport(StandardReport):
  name = "Task Tracker Report"
  extractors = [
    extractors.mr_failure_count,
    extractors.tt_last_seen,
    extractors.mr_running_reduces,
    extractors.mr_running_maps,
    ]

class DataNodeReport(StandardReport):
  name = "Data Node Report"
  extractors = [
    extractors.dn_last_seen,
    extractors.dn_disk_space,
    extractors.dn_avg_blockreport_time,
    extractors.dn_block_verification_failures,
    ]

class NameNodeReport(StandardReport):
  name = "Name Node Report"
  extractors = [
    extractors.nn_num_files,
    extractors.nn_under_replicated_blocks,
    extractors.nn_corrupt_blocks,
    extractors.nn_fsimage_load_time,
    ]


def get_reports_for_participant(participant, data):
  if participant.role == DATA_NODE:
    return [JvmReport(data), DataNodeReport(data)]
  elif participant.role == TASK_TRACKER:
    return [JvmReport(data), TaskTrackerReport(data)]
  elif participant.role == NAME_NODE:
    return [JvmReport(data), NameNodeReport(data)]
  elif participant.role == JOB_TRACKER:
    return [JvmReport(data)]
  else:
    return []
