# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
#
# Captures all datapoints from all clusters.

import time

from health.clusters import Cluster
from health import healthd_client
from threading import RLock

# Number of seconds to hold state for
REFRESH_INTERVAL = 2.0

class SystemHealthState(object):
  """
  Represents the overall health of the system, including all of its 
  registered clusters.  

  On refresh() (at reasonable intervals), this object accumulates data points,
  runs them through aggregations, and facilitates presenting data.
  """
  def __init__(self):
    self.data_points = []

  @classmethod
  def get_fresh_state(cls):
    state = cls()

    for cluster in Cluster.all():
      # Clusters are responsible for their own exception handling,
      # so we don't capture exceptions here.
      state.data_points += cluster.get_fresh_datapoints()

    state.data_points += healthd_client.get_fresh_datapoints()
    return state

  def get_participants(self):
    """Return a set of the participants in the cluster."""
    if hasattr(self, '_cache_participants'):
      return self._cache_participants

    parts = set()
    for dp in self.data_points:
      if dp.participant:
        parts.add(dp.participant)
    self._cache_participants = parts
    return parts


STATE = SystemHealthState()
STATE_REFRESHED_TS = 0 # Epoch time of last state refresh

STATE_LOCK = RLock()

def get_state():
  """Accessor for current health state that ensures thread safety
  and freshess of data."""
  global STATE

  STATE_LOCK.acquire()
  try:
    if time.time() - STATE_REFRESHED_TS > REFRESH_INTERVAL:
      STATE = SystemHealthState.get_fresh_state()
    return STATE
  finally:
    STATE_LOCK.release()
