#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
"""
Health monitoring application.

The state of the world is queried via the state SystemHealthState object.
The management of the data is handled by the health.state module.

The views are queries over the datapoints stored within the SystemHealthState.
Currently the query results are not cached, but this may be needed as a future
optimization.
"""

import itertools

from desktop.lib.django_util import render, MAKO
from health.models import HDFS, MR, GANGLIA, Participant
from health.clusters import Cluster
from health.state import get_state
from health.data import DataPoint
from health import query, lint
from health.cluster_views import GangliaClusterView, MRClusterView, HDFSClusterView, matches_role, per_state_count
from health import report
from health import extractors

from django.http import HttpResponse

def summary(request, template_file="summary.mako"):
  """
  Overall summary.  Per-cluster up/down/error states,
  and per-host up/down states.
  """

  # For every cluster, calculate the state counts:
  state = get_state()

  # Per-cluster state summaries.
  clusters = summarize_states_per_cluster(state.data_points)

  # Per-host state summaries
  def per_host(host, data):
    d = {}
    # Tuples are sorted lexicographically, so we put the sort key in the first
    # element.
    state_tuples = [x.key_value for x in data if x.key == DataPoint.STATE]
    if state_tuples:
      _, (min_state_k, min_state_v) = min([ (DataPoint.state_key(x.key_value), x.key_value) for x in data if x.key == DataPoint.STATE ])
    else:
      min_state_k, min_state_v = DataPoint.STATE_DOWN
    d[min_state_k] = min_state_v
    return host, d

  hosts = dict(query.query(data=state.data_points,
    select=per_host,
    where=lambda dp: dp.participant is not None,
    groupby=lambda dp: dp.participant.ip))

  up_down_host_counts = dict(query.query(data=hosts.values(),
    select=lambda state, hosts: (state, len(hosts)),
    groupby=lambda row: row[DataPoint.STATE]))
  up_down_host_counts.setdefault(DataPoint.STATE_UP[1], 0)
  up_down_host_counts.setdefault(DataPoint.STATE_DOWN[1], 0)
  up_down_host_counts.setdefault(DataPoint.STATE_ERROR[1], 0)

  lints = lint.run_all_lints(state.data_points)
  lint.sort_lints(lints)

  lints_by_level = {
    lint.CRITICAL: [],
    lint.WARN: [],
    lint.NOTICE: [],
    lint.OK: [],
    }
  for l in lints:
    lints_by_level[l.status].append(l)

  the_report = report.Report(state.data_points, extractors.get_heatmap_extractors())

  return render(
    template_file, request,
    dict(
      clusters=clusters,
      hosts=hosts,
      up_down_host_counts=up_down_host_counts,
      report=the_report,
      lints_by_level=lints_by_level),
    template_lib=MAKO)

def dashboard(request):
  return summary(request, 'dashboard.mako')

def dashboard_lints(request):
  return summary(request, 'dashboard.lints.mako')

def summarize_states_per_cluster(data):
  """
  Given datapoints, groups them by cluster, and then
  groups them by state.  It produces a histogram
  count of the states.
  """
  def per_cluster(cluster, data):
    def per_state(state_value, data):
      """Nested query that does a count"""
      return (state_value, len(data))
    return {
      "state_counts": per_state_count(data, None),
      "name": cluster.name,
      "type": cluster.type,
      "id": cluster.id
    }

  clusters = query.query(data=data,
    select=per_cluster,
    where=lambda dp: dp.cluster is not None,
    groupby=lambda dp: dp.cluster)

  return clusters

def data_points(request):
  """
  Debugging-oriented view showing all data points.
  """
  state = get_state()
  dp = state.data_points
  if request.GET.get("cluster"):
    dp = itertools.ifilter(lambda x: x.cluster and x.cluster.id == request.GET.get("cluster_name"), dp)
  if request.GET.get("participant"):
    dp = itertools.ifilter(lambda x: x.participant and x.participant.id == request.GET.get("participant"), dp)
    
  return render("data_points.html", request, { "data_points": dp })

def lints_summary(request):
  """
  Debugging-oriented view for all lint checks
  """
  state = get_state()
  lints = lint.run_all_lints(state.data_points)
  lint.sort_lints(lints)
  return render("lints.mako", request, {'lints': lints})

def ganglia(request):
  """
  Debugging-oriented view showing all ganglia points.
  """
  state = get_state()
  return render("ganglia.html", request, {
    "data_points": filter(matches_role("GMETAD"), state.data_points) 
  })

def cluster_id_is(id):
  """
  Returns a function that matches a specific cluster id.
  """
  def f(dp):
    return dp.cluster is not None and dp.cluster.id == id
  return f

def ip_is(ip):
  """
  Returns a function that matches a specific host IP
  """
  def f(dp):
    return dp.participant is not None and dp.participant.ip == ip
  return f

CLUSTER_TYPE_VIEW_MAP = {
  GANGLIA: GangliaClusterView,
  HDFS: HDFSClusterView,
  MR: MRClusterView,
}

def cluster(request, id):
  """
  Detailed view of a specific cluster.
  """
  id = id
  state = get_state()
  cluster = Cluster.get_by_id(id=id)
  data = filter(cluster_id_is(id), state.data_points)
  c = CLUSTER_TYPE_VIEW_MAP[cluster.type](data=data, cluster=cluster)
  return render("cluster.html", request, dict(
    name="%s(%s)" % (cluster.type, cluster.name),
    summary=c.render_summary(),
    participants=c.render_participants()))

def host(request, ip):
  """
  Detailed view of a host"""
  state = get_state()

  data = filter(ip_is(ip), state.data_points)

  def make_data_dict(participant, dps):
    return (participant, dict( (dp.key, dp) for dp in dps))

  by_participant = dict(
    query.query(data=state.data_points,
                select=make_data_dict,
                where=ip_is(ip),
                groupby=lambda dp: dp.participant))

  return render(
    "host.mako", request, dict(
      ip=ip,
      data_by_participant=by_participant))

def proxy(request):
  """
  Returns a list of available HTTP port proxies.
  """
  Participant.objects.all()[0].http_proxy
  return render("proxy.html", request, 
    dict(participants=Participant.objects.all()))

def heatmap_view(request):
  state = get_state()
  data = state.data_points
  the_report = report.Report(data, extractors.get_heatmap_extractors())
  return render("heatmap.mako", request, dict(report=the_report))


def update_state(request):
  state = get_state()
  return HttpResponse("OK")
update_state.login_notrequired = True
