#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
#
# Proxies HTTP requests through the Desktop server.
# This is intended to be used to view the "built-in"
# UIs.
#
# TODO(philip): Instead of whitelists, also offer a way
# to create links (within the application) to trusted
# URLs, by appending an HMAC to the parameters.

import logging
import re
from urllib2 import urlopen
from urlparse import urlparse, urlunparse

from django.core import urlresolvers
from django.http import HttpResponse
from desktop.lib.django_util import MessageException

from proxy import conf
import health.state

LOGGER = logging.getLogger(__name__)

def check_host_port(host, port):
  """
  Return true if this host:port pair is allowed to be proxied.
  """

  # Check explicit whitelist
  hostport = "%s:%d" % (host, port)
  for regexp in conf.WHITELIST.get():
    if regexp.match(hostport):
      return True

  # Check for daemons running in the cluster
  health_state = health.state.get_state()
  for participant in health_state.get_participants():
    # TODO(todd) Do we have to worry about DNS/canonicalization? At this point
    # we only proxy by IP so I think we're safe.
    if (host, port) == (participant.ip, participant.http_port):
      return True
  return False

def proxy(request, host, port, path):
  """
  Proxies an HTTP request by fetching the data
  and re-writing links.
  """
  port = int(port)

  if not check_host_port(host, port):
    raise MessageException(
      ("%s:%d is not whitelisted for reverse proxying, nor a daemon that Cluster Health " +
       "is aware of.  Contact your administrator.") % (host, port))

  # The tuple here is: (scheme, netloc, path, params, query, fragment).
  # We don't support params or fragment.
  url = urlunparse(("http", "%s:%d" % (host,port), 
                    path, 
                    None, 
                    request.META.get("QUERY_STRING"), 
                    None))
  LOGGER.info("Retrieving %s." % url)
  if request.method == 'POST':
    post_data = request.POST.urlencode()
  else:
    post_data = None
  data = urlopen(url, data=post_data)
  content_type = data.headers.get("content-type", "text/plain")
  if not re.match(r'^text/html\s*(?:;.*)?$', content_type):
    resp_text = data.read(1024*1024) # read 1MB
  else:
    resp_text = _rewrite_links(data)
  request.path = _reverse(host, port, path)
  return HttpResponse(resp_text, mimetype=data.headers.get("content-type"))

def _reverse(host, port, path):
  return urlresolvers.reverse("proxy.views.proxy",
                              kwargs=dict(host=host, port=port, path=path))
def _rewrite_url(url):
  """Used by _rewrite_links"""
  scheme, netloc, path, params, query, fragment = urlparse(url)
  if scheme != "http": # scheme
    # Only re-write http URLs, since that's all
    # we proxy.
    return url
  if ":" in netloc: # netloc
    host, port = netloc.split(":", 2)
  else:
    host, port = netloc, str(80)
  path = path or "/"
  out = _reverse(host, port, path)
  if query:
    out = out + "?" + query
  return out

def _rewrite_links(data):
  import lxml.html
  html = lxml.html.parse(data)
  html.getroot().rewrite_links(_rewrite_url, resolve_base_href=True, base_href=data.geturl())
  return lxml.html.tostring(html)
