#!/usr/bin/env python
# Copyright (c) 2010, Cloudera, inc.  All rights reserved.
#
# Implements simple file system browsing operations.
#
# Useful resources:
#   django/views/static.py manages django's internal directory index

import re
import logging
import mimetypes
import posixpath
import stat as stat_module
import urllib
import os

from django.core import urlresolvers
from django.http import Http404, HttpResponse, HttpResponseNotModified
from django.views.static import was_modified_since
from django.utils.http import http_date, urlquote
from django.utils.html import escape

from desktop.lib.django_util import make_absolute, render, render_json, MessageException, format_preserving_redirect
from filebrowser.lib.rwx import filetype, rwx
from filebrowser.lib import xxd
from filebrowser.forms import RenameForm, UploadForm, MkDirForm, RmDirForm, RmTreeForm, \
    RemoveForm, ChmodForm, ChownForm, EditorForm
from hadoop.fs import normpath

DEFAULT_CHUNK_SIZE_BYTES = 1024*4 # 4KB
MAX_CHUNK_SIZE_BYTES = 1024*1024 # 1MB
DOWNLOAD_CHUNK_SIZE = 32*1024 # 32KB

# Defaults for "xxd"-style output.
# Sentences refer to groups of bytes printed together, within a line.
BYTES_PER_LINE = 16
BYTES_PER_SENTENCE = 2
# If the percentage of non-printable bytes is greater than this, binary mode is
# enabled by default.
BINARY_PERCENTAGE = 0.10

# The maximum size the file editor will allow you to edit
MAX_FILEEDITOR_SIZE=256*1024

logger = logging.getLogger(__name__)

def _unquote_path(path):
  """Normalizes paths."""
  return urllib.unquote(path)

def _file_reader(fh):
  """Generator that reads a file, chunk-by-chunk."""
  while True:
    chunk = fh.read(DOWNLOAD_CHUNK_SIZE)
    if chunk == '':
      fh.close()
      break
    yield chunk

def download(request, path):
  """
  Downloads a file.

  This is inspired by django.views.static.serve.
  """
  path = _unquote_path(path)
  if not request.fs.exists(path):
    raise Http404("File not found: %s" % escape(path))
  if not request.fs.isfile(path):
    raise MessageException("Not a file", path)

  mimetype = mimetypes.guess_type(path)[0] or 'application/octet-stream'
  stats = request.fs.stats(path)
  mtime = stats['mtime']
  size = stats['size']
  if not was_modified_since(request.META.get('HTTP_IF_MODIFIED_SINCE'), mtime, size):
    return HttpResponseNotModified()
  # TODO(philip): Ideally a with statement would protect from leaks,
  # but tricky to do here.
  fh = request.fs.open(path)

  response = HttpResponse(_file_reader(fh), mimetype=mimetype)
  response["Last-Modified"] = http_date(stats['mtime'])
  response["Content-Length"] = stats['size']
  response["Content-Disposition"] = "attachment"
  return response

def view(request, path):
  """Dispatches viewing of a path to either index() or fileview(), depending on type."""
  if request.fs.isdir(path):
    return listdir(request, path)
  elif request.fs.isfile(path):
    return display(request, path)
  else:
    raise Http404("File not found: %s" % escape(path))

def edit(request, path, form=None):
  """Shows an edit form for the given path. Path does not necessarily have to exist."""
  
  try:
    stats = request.fs.stats(path)
  except IOError, ioe:
    # A file not found is OK, otherwise re-raise
    if "not found" in ioe.args[0]:
      stats = None
    else:
      raise

  # Can't edit a directory
  if stats and stats['mode'] & stat_module.S_IFDIR:
    raise MessageException("Path is a directory", path)

  # Maximum size of edit
  if stats and stats['size'] > MAX_FILEEDITOR_SIZE:
    raise MessageException("File too big to edit", path)

  if stats:
    f = request.fs.open(path)
    try:
      current_contents = f.read()
    finally:
      f.close()
  else:
    current_contents = ""

  if not form:
    form = EditorForm(dict(path=path, contents=current_contents))

  data = dict(
    exists=(stats is not None),
    form=form,
    path=path,
    filename=os.path.basename(path),
    dirname=os.path.dirname(path))
  return render("edit.mako", request, data)

def save_file(request):
  """
  The POST endpoint to save a file in the file editor.

  Does the save and then redirects back to the edit page.
  """
  form = EditorForm(request.POST)
  is_valid = form.is_valid()
  path = form.cleaned_data.get('path')
  
  if request.POST.get('save') == "saveAs":
    if not is_valid:
      return edit(request, path, form=form)
    else: 
      data = dict(form = form) 
      return render("saveas.mako", request, data)

  if not path:
    raise MessageException("No path specified")
  if not is_valid:
    return edit(request, path, form=form)

  if request.fs.exists(path):
    _do_overwrite_save(request.fs, path, form.cleaned_data['contents'])
  else:
    _do_newfile_save(request.fs, path, form.cleaned_data['contents'])
  
  request.flash.put('Saved %s.' % os.path.basename(path))
  return edit(request, path, form)

def _do_overwrite_save(fs, path, data):
  """
  Atomically (best-effort) save the specified data to the given path
  on the filesystem.

  TODO(todd) should this be in some fsutil.py?
  """
  # TODO(todd) Should probably do an advisory permissions check here to
  # see if we're likely to fail (eg make sure we own the file
  # and can write to the dir)

  # First write somewhat-kinda-atomically to a staging file
  # so that if we fail, we don't clobber the old one
  path_dest = path + "._cdesktop_new"

  new_file = fs.open(path_dest, "w")
  try:
    try:
      new_file.write(data)
      logging.info("Wrote to " + path_dest)
    finally:
      new_file.close()
  except Exception, e:
    # An error occurred in writing, we should clean up
    # the tmp file if it exists, before re-raising
    try:
      fs.remove(path_dest)
    except:
      pass
    raise e

  # Try to match the permissions and ownership of the old file
  cur_stats = fs.stats(path)
  try:
    fs.chmod(path_dest, cur_stats['mode'])
  except:
    logging.warn("Could not chmod new file %s to match old file %s" % (
        path_dest, path), exc_info=True)
    # but not the end of the world - keep going

  try:
    fs.chown(path_dest, cur_stats['user'], cur_stats['group'])
  except:
    logging.warn("Could not chown new file %s to match old file %s" % (
        path_dest, path), exc_info=True)
    # but not the end of the world - keep going

  # Now delete the old - nothing we can do here to recover
  fs.remove(path)

  # Now move the new one into place
  # If this fails, then we have no reason to assume
  # we can do anything to recover, since we know the
  # destination shouldn't already exist (we just deleted it above)
  fs.rename(path_dest, path)


def _do_newfile_save(fs, path, data):
  """
  Save data to the path 'path' on the filesystem 'fs'.

  There must not be a pre-existing file at that path.
  """
  new_file = fs.open(path, "w")
  try:
    new_file.write(data)
  finally:
    new_file.close()


def listdir(request, path, template="listdir.mako"):
  """
  Implements directory listing (or index).

  Intended to be called via view().
  """
  path = _unquote_path(path)
  if not request.fs.isdir(path):
    raise MessageException("Not a directory", path)

  file_filter = request.REQUEST.get('file_filter', 'any')

  assert file_filter in ['any', 'file', 'dir']

  data = {
    'path': path,
    'file_filter': file_filter,
    'stats': request.fs.stats(path),
    # This could also be put in automatically via
    # http://docs.djangoproject.com/en/dev/ref/templates/api/#django-core-context-processors-request,
    # but manually seems cleaner, since we only need it here.
    'current_request_path': request.path,
  }
  stats = request.fs.listdir_stats(path)
  # Include parent dir, unless at filesystem root.
  if normpath(path) != posixpath.sep:
    parent_stat = request.fs.stats(posixpath.join(path, ".."))
    # the 'path' field would be absolute, but we want its basename to be
    # actually '..' for display purposes
    parent_stat['path'] = posixpath.join(path, "..")
    stats.insert(0, parent_stat)

  data['files'] = [_massage_stats(request, stat) for stat in stats]
  return render(template, request, data)

def chooser(request, path):
  """
  Returns the html to JFrame that will display a file prompt.
  """
  return view(request, path)

def _massage_stats(request, stats):
  """
  Massage a stats record as returned by the filesystem implementation
  into the format that the views would like it in.
  """
  normalized = normpath(stats['path'])
  return {
    'path': normalized,
    'name': posixpath.basename(stats['path']),
    'stats': stats,
    'type': filetype(stats['mode']),
    'rwx': rwx(stats['mode']),
    'url': make_absolute(request, "view", dict(path=urlquote(normalized))),
  }

def stat(request, path):
  """
  Returns just the generic stats of a file.

  Intended for use via AJAX (and hence doesn't provide
  an HTML view).
  """
  path = _unquote_path(path)
  if not request.fs.exists(path):
    raise Http404("File not found: %s" % escape(path))
  stats = request.fs.stats(path)
  return render_json(_massage_stats(request, stats))

def display(request, path):
  """
  Implements displaying part of a file.

  GET arguments are length, offset, and mode, with reasonable
  defaults chosen.

  TODO(philip): Could easily built-in file type detection
  (perhaps using something similar to file(1)), as well
  as more advanced binary-file viewing capability (de-serialize
  sequence files, decompress gzipped text files, etc.).
  There exists a python-magic package to interface with libmagic.
  """
  path = _unquote_path(path)
  if not request.fs.isfile(path):
    raise MessageException("Not a file", path)

  stats = request.fs.stats(path)

  # I'm mixing URL-based parameters and traditional
  # HTTP GET parameters, since URL-based parameters
  # can't naturally be optional.
  length = int(request.GET.get("length", DEFAULT_CHUNK_SIZE_BYTES))
  # Display last block by default.
  offset = int(request.GET.get("offset", max(0, stats["size"] - (stats['size'] % length))))
  mode = request.GET.get("mode")
  if mode and mode not in ["binary", "text"]:
    raise MessageException("Mode must be one of 'binary' or 'text'.")
  if offset < 0:
    raise MessageException("Offset may not be less than zero.")
  if length < 0:
    raise MessageException("Length may not be less than zero.")
  if length > MAX_CHUNK_SIZE_BYTES:
    raise MessageException("Cannot request chunks greater than %d bytes" % MAX_CHUNK_SIZE_BYTES)

  f = request.fs.open(path)
  try:
    f.seek(offset)
    contents = f.read(length)
  finally:
    f.close()

  masked = None
  if not mode:
    # Auto-detect mode:
    (mask_count, masked) = xxd.mask_not_printable(contents)
    if mask_count and float(mask_count)/len(contents) > BINARY_PERCENTAGE:
      mode = "binary"
    else:
      mode = "text"

  if mode == "binary":
    xxd_out = list(xxd.xxd(offset, contents, BYTES_PER_LINE, BYTES_PER_SENTENCE))
  else:
    # May have been calculated already as part of detection.
    if not masked:
      (mask_count, masked) = xxd.mask_not_printable(contents)

  dirname = posixpath.dirname(path)
  # Start with index-like data:
  data = _massage_stats(request, request.fs.stats(path))
  # And add a view structure:
  data["success"] = True;
  data["view"] = {
    'offset': offset,
    'length': length,
    'end': offset + len(contents) - 1,
    'dirname': dirname,
    'mode': mode,
    'size': stats['size']
  }
  data["filename"] = os.path.basename(path) 
  data["editable"] = stats['size'] < MAX_FILEEDITOR_SIZE
  if mode == "binary":
    # This might be the wrong thing for ?format=json; doing the
    # xxd'ing in javascript might be more compact, or sending a less
    # intermediate representation...
    logger.debug("xxd: " + str(xxd_out))
    data['view']['xxd'] = xxd_out
    data['view']['masked_binary_data'] =  False
  else:
    data['view']['contents'] = masked
    data['view']['masked_binary_data'] = (mask_count > 0)

  return render("display.mako", request, data)

def _calculate_navigation(offset, length, size):
  """
  List of (offset, length, string) tuples for suggested navigation through the file.
  If offset is -1, then this option is already "selected".  (Whereas None would
  be the natural pythonic way, Django's template syntax doesn't let us test
  against None (since its truth value is the same as 0).)

  By all means this logic ought to be in the template, but the template
  language is too limiting.
  """
  if offset == 0:
    first, prev = (-1, None, "First Block"), (-1, None, "Previous Block")
  else:
    first, prev = (0, length, "First Block"), (max(0, offset - length), length, "Previous Block")

  if offset + length >= size:
    next, last = (-1, None, "Next Block"), (-1, None, "Last Block")
  else:
    # 1-off Reasoning: if length is the same as size, you want to start at 0.
    next, last = (offset + length, length, "Next Block"), (max(0, size - length), length, "Last Block")

  return first, prev, next, last

def generic_op(form_class, request, op, parameter_names, piggyback=None, template="fileop.mako"):
  """
  Generic implementation for several operations.

  @param form_class form to instantiate
  @param request incoming request, used for parameters
  @param op callable with the filesystem operation
  @param parameter_names list of form parameters that are extracted and then passed to op
  @param piggyback list of form parameters whose file stats to look up after the operation
  """
  # Use next for non-ajax requests, when available.
  next = request.GET.get("next")
  if next is None:
    next = request.POST.get("next")

  ret = dict({
    'next':next
  })
  for p in parameter_names:
    val = request.REQUEST.get(p)
    if val:
      ret[p] = val
  

  if request.method == 'POST':
    form = form_class(request.POST)
    # TODO(philip): How best to do error handling?  fs will throw
    # an arbitrary-ish exception (typically file not found or maybe permission
    # denied), and this needs to be coaxed into an HTTP error.
    ret['form'] = form
    if form.is_valid():
      args = [ form.cleaned_data[p] for p in parameter_names ]
      op(*args)
      if next:
        logging.debug("Next: %s" % next)
        # Doesn't need to be quoted: quoting is done by HttpResponseRedirect.
        return format_preserving_redirect(request, next)
      ret["success"] = True
      try:
        if piggyback:
          piggy_path = form.cleaned_data[piggyback]
          ret["result"] = _massage_stats(request, request.fs.stats(piggy_path))
      except Exception, e:
        # Hard to report these more naturally here.  These happen either
        # because of a bug in the piggy-back code or because of a
        # race condition.
        logger.exception("Exception while processing piggyback data")
        ret["result_error"] = True

      return render(template, request, ret)
  else:
    # Initial parameters may be specified with get
    initial_values = {}
    for p in parameter_names:
      val = request.GET.get(p)
      if val:
        initial_values[p] = val
    form = form_class(initial=initial_values)
    ret['form'] = form
  return render(template, request, ret)

def move(request):
  return generic_op(RenameForm, request, request.fs.rename, ["src_path", "dest_path"], None, template="move.mako")

def rename(request):
  def smart_rename(src_path, dest_path):
    """If dest_path doesn't have a directory specified, use same dir."""
    if "/" not in dest_path:
      src_dir = os.path.dirname(src_path)
      dest_path = os.path.join(src_dir, dest_path)
    request.fs.rename(src_path, dest_path)
  return generic_op(RenameForm, request, smart_rename, ["src_path", "dest_path"], None, template="rename.mako")

def mkdir(request):
  def smart_mkdir(path, name):
    request.fs.mkdir(os.path.join(path, name))
  return generic_op(MkDirForm, request, smart_mkdir, ["path", "name"], "path", template="mkdir.mako")

def remove(request):
  return generic_op(RemoveForm, request, request.fs.remove, ["path"], None)

def rmdir(request):
  return generic_op(RmDirForm, request, request.fs.rmdir, ["path"], None)

def rmtree(request):
  return generic_op(RmTreeForm, request, request.fs.rmtree, ["path"], None)

def chmod(request):
  # mode here is abused: on input, it's a string, but when retrieved,
  # it's an int.
  return generic_op(ChmodForm, request, request.fs.chmod, ["path", "mode"], "path", template="chmod.mako")

def chown(request):
  return generic_op(ChownForm, request, request.fs.chown, ["path", "user", "group"], "path", template="chown.mako")

def upload(request):
  """
  Handles file uploads.

  Django has a hook for "upload handlers" that could make this more
  efficient.  Currently Django will store files greater than 2.5MB in
  a temp directory.  We could implement an upload handler to
  send data right through to the destination.

  http://docs.djangoproject.com/en/1.0/topics/http/file-uploads/#upload-handlers
  """
  if request.method == 'POST':
    form = UploadForm(request.POST, request.FILES)
    if form.is_valid():
      # Bit of a wart that form.file doesn't give you the file,
      # and you have to do form.files.get("file").
      file = form.files.get("file")
      dest = form.cleaned_data["dest"]
      if request.fs.isdir(dest):
        assert posixpath.sep not in file.name
        dest = posixpath.join(dest, file.name)
      output = request.fs.open(dest, "w")
      try:
        for chunk in file.chunks():
          output.write(chunk)
      finally:
        output.close()

      dest_stats = request.fs.stats(dest)
      return render('upload_done.mako', request, {
          # status is used by "fancy uploader"
          'status': 1,
          'path': dest,
          'result': _massage_stats(request, dest_stats),
          'next': request.GET.get("next")
      })
  else:
    dest = request.GET.get("dest")
    initial_values = {}
    if dest:
      initial_values["dest"] = dest
    form = UploadForm(initial=initial_values)
  return render('upload.mako', request, {'form': form, 'next': request.REQUEST.get("dest")})

def status(request):
  status = request.fs.status()
  data = {
    # Beware: "messages" is special in the context browser.
    'msgs': status.get_messages(),
    'health': status.get_health(),
    'datanode_report': status.get_datanode_report(),
    'name': request.fs.name
  }
  return render("status.mako", request, data)

def return_template(request, tmpl):
  return render(tmpl, request, {})

HDFS_NO_PORT = re.compile("^hdfs://([^/:]+)(/.*)$")
DEFAULT_HDFS_PORT = 8020
def location_to_url(request, location):
  """
  If possible, returns a file browser URL to the location.
  Location is a URI.

  Python doesn't seem to have a readily-available URI-comparison
  library, so this is quite hacky.
  """
  path = location_to_path(request, location)
  if  path != None:
    return urlresolvers.reverse("filebrowser.views.view", kwargs=dict(path=path))
  else:
    return None

def location_to_path(request, location):
  """
  Returns an HDFS path compatible with HadoopFS object.
  """
  if location.startswith(request.fs.uri):
    return location[len(request.fs.uri):]
  elif HDFS_NO_PORT.match(location):
    m = HDFS_NO_PORT.match(location)
    return m.groups()[1]
  else:
      return None

def truncate(toTruncate, charsToKeep=50):
  """
  Returns a string truncated to 'charsToKeep' length plus ellipses.
  """
  if len(toTruncate) > charsToKeep:
    truncated = toTruncate[:charsToKeep] + "..."
    return truncated
  else:
    return toTruncate 

