#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
#
# Tests for "user manager"

from nose.tools import assert_true, assert_equal, assert_false
from nose.plugins.attrib import attr

from desktop.lib.django_test_util import make_logged_in_client
from django.contrib.auth.models import User
from hadoop import mini_cluster

def reset_all_users():
  """Reset to a clean state by deleting all users"""
  for user in User.objects.all():
    user.delete()

def test_user_manager():
  reset_all_users()
  c = make_logged_in_client(username="test", is_superuser=True)

  # Test basic output.
  response = c.get('/userman/')
  assert_true(len(response.context["users"]) > 0)
  assert_true("Cloudera Desktop Users" in response.content)

  # Test editing a superuser
  # Just check that this comes back
  response = c.get('/userman/edit/test')
  # Edit it, to add a first and last name
  response = c.post('/userman/edit/test', dict(username="test", first_name="Tom", last_name="Tester", is_superuser="True", is_active="True"))
  # Now make sure that those were materialized
  response = c.get('/userman/edit/test')
  assert_equal("Tom", response.context["form"].instance.first_name)
  assert_true("Tester" in response.content)
  # Shouldn't be able to demote to non-superuser
  response = c.post('/userman/edit/test', dict(username="test",
                        first_name="Tom", last_name="Tester",
                        is_superuser=False, is_active=True))
  assert_true("You cannot remove" in response.content,
              "Shouldn't be able to remove the last superuser")
  # Let's try changing the password
  response = c.post('/userman/edit/test', dict(username="test", first_name="Tom", last_name="Tester", is_superuser=True, password1="foo", password2="foobar"))
  assert_equal(["Passwords do not match."], response.context["form"]["password2"].errors, "Should have complained about mismatched password")
  response = c.post('/userman/edit/test', dict(username="test", first_name="Tom", last_name="Tester", password1="foo", password2="foo", is_active=True, is_superuser=True))
  assert_true(User.objects.get(username="test").is_superuser)
  assert_true(User.objects.get(username="test").check_password("foo"))
  # Change it back!
  response = c.post('/userman/edit/test', dict(username="test", first_name="Tom", last_name="Tester", password1="test", password2="test", is_active="True", is_superuser="True"))
  assert_true(User.objects.get(username="test").check_password("test"))
  assert_true(make_logged_in_client(username = "test", password = "test"),
              "Check that we can still login.")

  # Create a new regular user (duplicate name)
  assert_true(c.get('/userman/new'))
  response = c.post('/userman/new', dict(username="test", password1="test", password2="test"))
  assert_equal({ 'username': ["User with this Username already exists."]}, response.context["form"].errors)

  # Create a new regular user (for real)
  response = c.post('/userman/new', dict(username="test_user_creation", password1="test", password2="test", is_active="True"))
  response = c.get('/userman/')
  assert_true("test_user_creation" in response.content)
  assert_true(len(response.context["users"]) > 1)
  assert_true("Cloudera Desktop Users" in response.content)

  # Check permissions by logging in as the new user
  c_reg = make_logged_in_client(username="test_user_creation", password="test")
  # Regular user should be able to modify oneself
  response = c_reg.post('/userman/edit/test_user_creation',
                        dict(username = "test_user_creation",
                        first_name = "Hello",
                        is_active = True))
  response = c_reg.get('/userman/edit/test_user_creation')
  assert_equal("Hello", response.context["form"].instance.first_name)
  # Can't edit other people.
  response = c_reg.post("/userman/delete/test")
  assert_true("You must be a superuser" in response.content,
              "Regular user can't edit other people")
  # Regular user should not be able to self-promote to superuser
  response = c_reg.post('/userman/edit/test_user_creation',
                        dict(username = "test_user_creation",
                        first_name = "Hello",
                        is_superuser = True,
                        is_active = True))
  assert_true("You cannot" in response.content,
              "Regular users can't self-promote to superuser")

  # Revert to regular "test" user, that has superuser powers.
  c_su = make_logged_in_client()
  # Inactivate "test_user_creation"
  c_su.post('/userman/edit/test_user_creation',
                        dict(username = "test_user_creation",
                        first_name = "Hello",
                        is_active = False))
  # Now make sure "test_user_creation" can't log back in
  response = c_reg.get('/userman/edit/test_user_creation')
  assert_true(response.status_code == 302 and "login" in response["location"],
              "Inactivated user gets redirected to login page")

  # Delete that regular user
  response = c_su.post('/userman/delete/test_user_creation')
  assert_true("Cloudera Desktop Users" in response.content)
  # You shouldn't be able to create a user without a password
  response = c_su.post('/userman/new', dict(username="test"))
  assert_true("You must specify a password when creating a new user." in response.content)


@attr('requires_hadoop')
def test_home_dir_management():
  """
  Tries permutations of creating and deleting home dirs.
  """
  reset_all_users()
  c = make_logged_in_client()
  cluster = mini_cluster.shared_cluster(conf=True)
  try:
    # Delete if there's already something there
    cluster.fs.setuser(cluster.superuser)
    if cluster.fs.isdir("/user/test"):
      cluster.fs.rmtree("/user/test")

    # Let's check that test's home dir isn't created
    response = c.post('/userman/')
    assert "test" not in response.context["homedirs"]
    # Let's create the home dir
    response = c.post("/userman/create_home/test")
    assert_true("test" in response.context["homedirs"])
    stats = cluster.fs.stats("/user/test")
    assert_equal("test", stats["user"])
    assert_equal("test", stats["group"])
    # Let's make sure it's not available in the edit screen
    response = c.get("/userman/edit/test")
    assert_false("create_home" in response.content)
    # Delete it
    cluster.fs.setuser(cluster.superuser)
    cluster.fs.rmdir("/user/test")
    # It should be in the edit screen
    response = c.get("/userman/edit/test")
    assert_true("create_home" in response.content)
    # Let's make sure it works in the edit screen
    response = c.post("/userman/edit/test", dict(username="test", first_name="Tom", last_name="Tester", password1="test", password2="test", is_active=True, is_superuser=True, create_home=True))
    assert_true("test" in response.context["homedirs"])
    stats = cluster.fs.stats("/user/test")
    assert_equal("test", stats["user"])
    # Delete it again, to clean up state
    cluster.fs.setuser(cluster.superuser)
    cluster.fs.rmdir("/user/test")
    # Let's make sure it works for new users
    response = c.post("/userman/new", dict(username="test_new", first_name="Tom", last_name="Tester", password1="test", password2="test", is_active=True, is_superuser=True, create_home=True))
    assert_true("test_new" in response.context["homedirs"])
    cluster.fs.rmdir("/user/test_new")
  finally:
    cluster.fs.setuser("webui", ["webui"])
    cluster.shutdown()
