#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
"""
Authentication backend classes for the File Browser.

These classes should implement the interface described at:
http://docs.djangoproject.com/en/1.0/topics/auth/#writing-an-authentication-backend
"""
from django.contrib.auth.models import User
import django.contrib.auth.backends
import logging

LOG = logging.getLogger(__name__)

def find_or_create_user(username, password=None):
  try:
    user = User.objects.get(username=username)
    LOG.debug("Found user %s in the db" % username)
  except User.DoesNotExist:
    LOG.info("Materializing user %s in the database" % username)
    user = User(username=username)
    if password is None:
      user.set_unusable_password()
    else:
      user.set_password(password)
    user.is_superuser = True
    user.save()
  return user


class DesktopBackendBase(object):
  """
  Abstract base class for providing external authentication schemes.

  Extend this class and implement check_auth
  """
  def authenticate(self, username, password):
    if self.check_auth(username, password):
      return find_or_create_user(username)
    else:
      return None

  def get_user(self, user_id):
    try:
      user = User.objects.get(pk=user_id)
      return user
    except User.DoesNotExist:
      return None

  def check_auth(self, username, password):
    """
    Implementors should return a boolean value which determines
    whether the given username and password pair is valid.
    """
    raise Exception("Abstract class - must implement check_auth")

class AllowFirstUserDjangoBackend(django.contrib.auth.backends.ModelBackend):
  """
  Allows the first user in, but otherwise delegates to Django's
  ModelBackend.
  """
  def authenticate(self, username=None, password=None):
    user = super(AllowFirstUserDjangoBackend, self).authenticate(username, password)
    if user is not None:
      if user.is_active:
        return user
      return None

    if self.is_first_login_ever():
      return find_or_create_user(username, password)

    return None

  def is_first_login_ever(self):
    """ Return true if no one has ever logged in to Desktop yet. """
    return User.objects.count() == 0

class AllowAllBackend(DesktopBackendBase):
  """
  Authentication backend that allows any user to login as long
  as they have a username and password of any kind.
  """
  def check_auth(self, username, password):
    return True

class PamBackend(DesktopBackendBase):
  """
  Authentication backend that uses PAM to authenticate logins.
  """
  def check_auth(self, username, password):
    import pam
    return pam.authenticate(username, password)
