#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
"""General configuration for core Desktop features (authentication, etc)"""

from desktop.lib.conf import Config, ConfigSection, UnspecifiedConfigSection, coerce_bool
from desktop.lib.paths import get_desktop_root
import os
import socket

HTTP_HOST = Config(
  key="http_host",
  help="HTTP Host to bind to",
  type=str,
  default="0.0.0.0")
HTTP_PORT = Config(
  key="http_port",
  help="HTTP Port to bind to",
  type=int,
  default=8088)
SSL_CERTIFICATE = Config(
  key="ssl_certificate",
  help="Filename of SSL Certificate",
  default=None)
SSL_PRIVATE_KEY = Config(
  key="ssl_private_key",
  help="Filename of SSL RSA Private Key",
  default=None)
ENABLE_CHERRYPY_SERVER = Config(
  key="enable_cherrypy_server",
  help="If set to false, runcpserver will not actually start the web server.  Used if Apache is being used as a WSGI container.",
  type=coerce_bool,
  default=True)
CHERRYPY_SERVER_THREADS = Config(
  key="cherrypy_server_threads",
  help="Number of threads used by the CherryPy web server.",
  type=int,
  default=10)

#
# Email (SMTP) settings
#
def default_from_email():
  """Email for cdesktop@<host-fqdn>"""
  try:
    fqdn = socket.getfqdn()
  except:
    fqdn = 'localhost'
  return "cdesktop@" + fqdn

SMTP = ConfigSection(
  key='smtp',
  help='Configuration options for connecting to an external SMTP server',
  members=dict(
    HOST = Config(
      key="host",
      help="The SMTP server for email notification delivery",
      type=str,
      default="localhost"
    ),

    PORT = Config(
      key="port",
      help="The SMTP server port",
      type=int,
      default=25
    ),

    USER = Config(
      key="user",
      help="The username for the SMTP host",
      type=str,
      default=""
    ),

    PASSWORD = Config(
      key="password",
      help="The password for the SMTP user",
      type=str,
      default=""
    ),

    USE_TLS = Config(
      key="tls",
      help="Whether to use a TLS (secure) connection when talking to the SMTP server",
      type=coerce_bool,
      default=False
    ),

    DEFAULT_FROM= Config(
      key="default_from_email",
      help="Default email address to use for various automated notification from Cloudera Desktop",
      type=str,
      dynamic_default=default_from_email
    ),
  )
)

DATABASE = ConfigSection(
  key='database',
  help="""Configuration options for specifying the Desktop Database.
          For more info, see http://docs.djangoproject.com/en/1.1/ref/settings/#database-engine""",
  members=dict(
    ENGINE=Config(
      key='engine',
      help='Database engine, eg postgresql, mysql, sqlite3, or oracle',
      type=str,
      default='sqlite3',
    ),
    NAME=Config(
      key='name',
      help='Database name, or path to DB if using sqlite3',
      type=str,
      default=get_desktop_root('desktop.db'),
    ),
    USER=Config(
      key='user',
      help='Database username',
      type=str,
      default='',
    ),
    PASSWORD=Config(
      key='password',
      help='Database password',
      type=str,
      default='',
    ),
    HOST=Config(
      key='host',
      help='Database host',
      type=str,
      default='',
    ),
    PORT=Config(
      key='port',
      help='Database port',
      type=int,
      default=0,
    ),
  )
)

# See python's documentation for time.tzset for valid values.
TIME_ZONE = Config(
  key="time_zone",
  help="Time zone name",
  type=str,
  default=os.environ.get("TZ", "America/Los_Angeles")
)

SERVER_USER = Config(
  key="server_user",
  help="Username to run servers as",
  type=str,
  default="cdesktop")
SERVER_GROUP = Config(
  key="server_group",
  help="Group to run servers as",
  type=str,
  default="cdesktop")


AUTH = ConfigSection(
  key="auth",
  help="Configuration options for user authentication into the web application",
  members=dict(
    BACKEND=Config("backend",
                   default="desktop.auth.backend.AllowFirstUserDjangoBackend",
                   help="Authentication backend.  Common settings are "
                        "django.contrib.auth.backends.ModelBackend (fully Django backend), " + 
                        "desktop.auth.backend.AllowAllBackend (allows everyone), " +
                        "desktop.auth.backend.AllowFirstUserDjangoBackend (relies on Django and user manager, after the first login)")
    ))

LOCAL_FILESYSTEMS = UnspecifiedConfigSection(
  key="local_filesystems",
  help="Paths on the local file system that users should be able to browse",
  each=ConfigSection(
    members=dict(
      PATH=Config("path",
                  required=True,
                  help="The path on the local FS"))))

def default_feedback_url():
  """A version-specific URL."""
  # Do this lazily to avoid loop
  from django.conf import settings
  return "http://tiny.cloudera.com/desktop-feedback-%s" % settings.CLOUDERA_DESKTOP_VERSION
  
FEEDBACK_URL = Config(
  key="feedback_url",
  help="Link for 'feedback' tab.",
  type=str,
  dynamic_default=default_feedback_url
)
