#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
#
# Utilities for django operations.

import logging
import simplejson
import re

from django.core import urlresolvers, serializers
from django.conf import settings
from django.http import QueryDict, HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response as django_render_to_response
from django.template.loader import render_to_string as django_render_to_string
from django.template import RequestContext
from django.db import models
from desktop.lib import django_mako

import thrift_util

# Values for template_lib parameter
DJANGO = 'django'
MAKO = 'mako'

class Encoder(simplejson.JSONEncoder):
  """
  Automatically encodes JSON for Django models and
  Thrift objects, as well as objects that have
  "to_json" operations.
  """
  def default(self, o):
    if hasattr(o, "to_jsonable") and callable(o.to_jsonable):
      return o.to_jsonable()
    if thrift_util.is_thrift_struct(o):
      return thrift_util.thrift2json(o)
    if isinstance(o, models.Model):
      # Django serialization requires an iterable, so we wrap o in an array.
      x = serializers.serialize("python", [o])
      assert len(x) == 1
      return x[0]

    return simplejson.JSONEncoder.default(self, o)


def copy_query_dict(query_dict, attr_list):
  """
  copy_query_dict(query_dict ,attr_list) -> QueryDict object
  Copy the specified attributes to a new QueryDict.
  """
  res = QueryDict('', mutable=True)
  for attr in attr_list:
    if query_dict.has_key(attr):
      res[attr] = query_dict.get(attr)
  return res


def make_absolute(request, view_name, kwargs=None):
  """
  Magic to make an absolute url.  In the template world,
  this is done with {% url %}.
  """
  return request.build_absolute_uri(urlresolvers.reverse(view_name, kwargs=kwargs))

def _get_template_lib(template, kwargs):
  template_lib = kwargs.get('template_lib')
  if 'template_lib' in kwargs:
    del kwargs['template_lib']

  # Default based on file extension
  if template_lib == None:
    if template.endswith('.mako'):
      return MAKO
    else:
      return DJANGO
  return template_lib


def _render_to_response(template, *args, **kwargs):
  template_lib = _get_template_lib(template, kwargs)

  if template_lib == DJANGO:
    return django_render_to_response(template, *args, **kwargs)
  elif template_lib == MAKO:
    return django_mako.render_to_response(template, *args, **kwargs)
  else:
    raise Exception("Bad template lib: %s" % template_lib)

def render_to_string(template, *args, **kwargs):
  """Wrapper around django.template.loader.render_to_string which supports
  different template libraries."""
  template_lib = _get_template_lib(template, kwargs)
  if template_lib == DJANGO:
    return django_render_to_string(template, *args, **kwargs)
  elif template_lib == MAKO:
    return django_mako.render_to_string(template, *args, **kwargs)
  else:
    raise Exception("Bad template lib: %s" % template_lib)

def format_preserving_redirect(request, target, get_dict=None):
  """
  If request represents an ajax or embeddable "format", tries
  to preserve that for redirects.

  Any GET params may be passed through the QueryDict get_dict,
  which should be mutable.
  """
  my_get_dict = QueryDict('', mutable=True)
  if get_dict:
    my_get_dict.update(get_dict)

  if is_jframe_request(request):
    logging.info("JFrame redirection" +  target)
    my_get_dict['format'] = 'embed'
  elif request.ajax:
    my_get_dict['format'] = 'json'

  param = my_get_dict.urlencode()
  if param:
    param = '?' + param
  return HttpResponseRedirect(target + param)

def is_jframe_request(request):
  """
  The JFrame container uses ?format=embed to request
  embeddable contents, and expects an HTTP response
  with some extra headers set in return.
  The extra headers are set in JFrameMiddleware.
  See also CCS.JFrame.js.
  """
  return request.META.get('HTTP_X_CLOUDERA_JFRAME') or \
      request.GET.get("format") == "embed"

def render(template, request, data, json=None, template_lib=None, force_template=False, **kwargs):
  """
  Render() is the main shortcut/workhorse for rendering view responses.
  It takes a template (either ".mako" or ".html", or influenced by
  template_lib), as well as as arbitrary data.

  It typically renders to an HttpResponse.  If the request is a non-JFrame
  AJAX reqeust (or if data is None), it renders into JSON.

  if force-template is True, will render the non-AJAX template response even if the
  request is via AJAX. This is to facilitate fetching HTML fragments.
  """
  if not force_template and not is_jframe_request(request) and (request.ajax or template is None):
    if json is not None:
      return render_json(json, request.GET.get("callback"))
    else:
      return render_json(data, request.GET.get("callback"))
  else:
    return _render_to_response(template,
                               RequestContext(request=request, dict=data),
                               template_lib=template_lib,
                               **kwargs)

def encode_json(data, indent=None):
  """
  Converts data into a JSON string.

  Typically this is used from render_json, but it's the natural
  endpoint to test the Encoder logic, so it's separated out.
  """
  return simplejson.dumps(data, indent=indent, cls=Encoder)

VALID_JSON_IDENTIFIER = re.compile("^[a-zA-Z_$][a-zA-Z0-9_$]*$")

class IllegalJsonpCallbackNameException(Exception):
  pass

def render_json(data, jsonp_callback=None):
  """
  Renders data as json.  If jsonp is specified, wraps
  the result in a function.
  """
  if settings.DEBUG:
    indent = 2
  else:
    indent = 0
  json = encode_json(data, indent)
  if jsonp_callback is not None:
    if not VALID_JSON_IDENTIFIER.match(jsonp_callback):
      raise IllegalJsonpCallbackNameException("Invalid jsonp callback name: %s" % jsonp_callback)
    json = "%s(%s);" % (jsonp_callback, json)
  return HttpResponse(json, mimetype='text/javascript')

def update_if_dirty(model_instance, **kwargs):
  """
  Updates an instance of a model with kwargs.
  saves only if there's been a change.
  """
  dirty = False
  for key, value in kwargs.items():
    if getattr(model_instance, key) != value:
      setattr(model_instance, key, value)
      dirty = True

  if dirty:
    model_instance.save()

class StructuredException(Exception):
  """
  Many exceptions in this application are a string and a filename
  that applies to.  The middleware will take these exceptions
  and render them.
  """
  def __init__(self, code, message, data=None):
    Exception.__init__(self, message)
    self.code = code
    self.message = message
    self.data = data

  def __str__(self):
    return "%s (code %s): %s" % (self.message, self.code, repr(self.data))

  @property
  def response_data(self):
    return dict(code=self.code,
                message=self.message,
                data=self.data)

class MessageException(StructuredException):
  """
  Explicitly specified msg/filename exception.

  This has been superceded by PopupException.
  """
  def __init__(self, msg, filename=None):
    StructuredException.__init__(self,
      code="GENERIC_MESSAGE",
      message=msg,
      data=dict(filename=filename))

class PopupException(Exception):
  """
  Middleware will render this exception; and the template
  renders it as a pop-up.
  """
  def __init__(self, message, title="Error", detail=None):
    Exception.__init__(self, message)
    self.message = message
    self.title = title
    self.detail = detail

  def response(self, request):
    return render("popup_error.mako", request, 
      dict(title=self.title, message=self.message, detail=self.detail, request=request))
