# Tests for django_util
# Some parts based on http://www.djangosnippets.org/snippets/1044/
# Copyright (c) 2010, Cloudera, inc. All rights reserved.

import unittest

from desktop.lib.django_test_util import configure_django_for_test, create_tables
configure_django_for_test()

from desktop.lib import django_util
from django.db import models

class TestModel(models.Model):
  class Meta:
    app_label = "TEST_APP"

  my_int = models.IntegerField()
  my_str = models.TextField(max_length=100)
  last_modified = models.DateField(auto_now=True)

class TestDjangoUtil(unittest.TestCase):
  def test_update_if_dirty(self):
    """
    Tests that update_if_dirty works.
    We use the last_modified field as a proxy for knowing
    whether or not we actually did a save.
    """
    create_tables(TestModel)
    x = TestModel()
    x.my_int = 3
    x.my_string = "foo"
    x.save()
    last_mod = x.last_modified

    django_util.update_if_dirty(x, my_int=3, my_string="bar")
    self.assertNotEquals(x.last_modified, last_mod)
    last_mod = x.last_modified

    django_util.update_if_dirty(x, my_int=3, my_string="bar")
    self.assertEquals(x.last_modified, last_mod)

    x.delete()

  def test_encode_json_unrenderable(self):
    class Foo(object):
      pass
    self.assertRaises(TypeError, django_util.encode_json, [ Foo() ])

  def test_encode_json_model(self):
    self.assertEquals('{"pk": null, "model": "TEST_APP.testmodel", "fields": {"last_modified": null, "my_str": "foo", "my_int": 3}}',
        django_util.encode_json(TestModel(my_int=3, my_str="foo")))
    self.assertEquals('[{"pk": null, "model": "TEST_APP.testmodel", "fields": {"last_modified": null, "my_str": "foo", "my_int": 3}}]',
        django_util.encode_json([TestModel(my_int=3, my_str="foo")]))

  def test_encode_json_to_jsonable(self):
    class Foo(object):
      def to_jsonable(self):
        return "foo"
    self.assertEquals('"foo"', django_util.encode_json(Foo()))
    self.assertEquals('["foo", "foo"]', django_util.encode_json([Foo(), Foo()]))
    self.assertEquals('{"pk": null, "model": "TEST_APP.testmodel", "fields": {"last_modified": null, "my_str": "foo", "my_int": 3}}',
        django_util.encode_json(TestModel(my_int=3, my_str="foo")))

    class Bar(object):
      to_jsonable = "not a callable"
    self.assertRaises(TypeError, django_util.encode_json, [ Bar() ])

  def test_encode_json_thrift(self):
    # TODO(philip): I've avoided writing this because
    # I don't want to modify sys.path to include the gen-py stuff.
    pass

  def test_render_json_jsonp(self):
    self.assertEquals("foo(3);", django_util.render_json(3, jsonp_callback="foo").content)

  def test_render_json_jsonp_bad_name(self):
    # Bad names
    for x in [r"%evil-name", "3vil", "", "evil%"]:
      self.assertRaises(django_util.IllegalJsonpCallbackNameException,
        django_util.render_json, "whatever-value", x)
    # Fine names
    for x in ["a", "$", "_", "a9", "a9$"]:
      django_util.render_json("whatever-value", x)


  def test_exceptions(self):
    msg = "b0rked file"
    the_file = "foobar"
    try:
      raise django_util.MessageException(msg, the_file)
    except Exception, e:
      self.assertEquals(msg, e.message)
      self.assertEquals(the_file, e.data['filename'])
      self.assertTrue(msg in str(e))
if __name__ == '__main__':
  unittest.main()
