#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
""" Basic setup functions for logging """

import logging
import os

LOG_FORMAT='%(asctime)s %(module)-12s %(levelname)-8s %(message)s'

def get_stderr_log_level():
  """
  Determine the log level that we should print out to stderr.

  The log level may be specified in the DESKTOP_LOGLEVEL environment
  variable. If that is not specified, then defaults to INFO, unless
  the DESKTOP_DEBUG environment is present, in which case it
  defaults to DEBUG.
  """
  env_setting = os.getenv("DESKTOP_LOGLEVEL")
  if env_setting:
    try:
      return getattr(logging, env_setting.upper())
    except AttributeError:
      raise Exception("Invalid log level in DESKTOP_LOGLEVEL: %s" % env_setting)

  if os.getenv("DESKTOP_DEBUG"):
    return logging.DEBUG
  return logging.INFO

def basic_logging():
  """
  Like logging.basicConfig, but explicit.

  This removes all previously installed logging handlers.
  """

  root_logger = logging.getLogger()

  # First get rid of any preinstalled/default handlers
  for h in root_logger.handlers:
    root_logger.removeHandler(h)

  # always keep DEBUG at the root, since we'll filter in the
  # handlers themselves - this allows the /logs endpoint
  # to always have all logs.
  root_logger.setLevel(logging.DEBUG)

  # Make a basic handler that prints to stderr
  stream_handler = logging.StreamHandler()
  stream_handler.setLevel(get_stderr_log_level())
  stream_handler.setFormatter(logging.Formatter(LOG_FORMAT))
  root_logger.addHandler(stream_handler)

def fancy_logging():
  """Configure logging into a buffer for /logs endpoint."""
  from desktop.lib.log_buffer import FixedBufferHandler
  BUFFER_SIZE = 10*1024*1024 # This is the size in characters, not bytes
  buffer_handler = FixedBufferHandler(BUFFER_SIZE)
  formatter = logging.Formatter(LOG_FORMAT)
  # We always want to catch all messages in our error report buffer
  buffer_handler.setLevel(logging.DEBUG)
  buffer_handler.setFormatter(formatter)
  root_logger = logging.getLogger()
  root_logger.addHandler(buffer_handler)

