#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
#
# Django settings for Cloudera Desktop.
#
# Local customizations are done by symlinking a file
# as local_settings.py.

import logging
import os
import desktop.conf
from desktop import logging_setup
from desktop.lib.paths import get_desktop_root
import pkg_resources

CLOUDERA_DESKTOP_VERSION = pkg_resources.get_distribution("desktop").version or "Unknown"
NICE_NAME = "Cloudera Desktop"

############################################################
# Part 1: Logging and imports.
############################################################

# Configure debug mode
DEBUG = True
TEMPLATE_DEBUG = DEBUG

# Start basic logging as soon as possible.
logging_setup.basic_logging()

logging.info("Welcome to Cloudera Desktop " + CLOUDERA_DESKTOP_VERSION)

# Then we can safely import some more stuff
from desktop import appmanager
from desktop.lib import conf

# Add fancy logging
logging_setup.fancy_logging()


############################################################
# Part 2: Generic Configuration
############################################################
ADMINS = (
    ('Cloudera Administrator', 'admin@cloudera.com')
)
MANAGERS = ADMINS

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'en-us'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = False

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash if there is a path component (optional in other cases).
# Examples: "http://media.lawrence.com", "http://example.com/media/"
MEDIA_URL = ''

# URL prefix for admin media -- CSS, JavaScript and images. Make sure to use a
# trailing slash.
# Examples: "http://foo.com/media/", "/media/".
ADMIN_MEDIA_PREFIX = '/media/'

# Make this unique, and don't share it with anybody.
SECRET_KEY = '&a_=0^vg+53ed5sb+8v771nc1=-&s7o#e$&+3@2n2k$gq%58!1'

############################################################
# Part 3: Django configuration
############################################################

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.load_template_source',
    'desktop.lib.template_loader.load_template_source',
)

MIDDLEWARE_CLASSES = [
    'django.middleware.common.CommonMiddleware',
    'desktop.middleware.SessionOverPostMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'desktop.middleware.AjaxMiddleware',
    # Must be after Session, Auth, and Ajax.  Before everything else.
    'desktop.middleware.RequireLoginEverywhereMiddleware',
    'desktop.middleware.FlashMessageMiddleware',
    'desktop.middleware.JFrameMiddleware',
    'desktop.middleware.ExceptionMiddleware',
    'desktop.middleware.ClusterMiddleware',
    'desktop.middleware.AppSpecificMiddleware',

    # 'debug_toolbar.middleware.DebugToolbarMiddleware'
]

ROOT_URLCONF = 'desktop.urls'

TEMPLATE_DIRS = (
    get_desktop_root("core/templates")
)

INSTALLED_APPS = [
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',

    'django.contrib.admin',
    'django_extensions',
    # 'debug_toolbar',

    # Desktop injects all the other installed apps into here magically.
    'desktop'
]

AUTH_PROFILE_MODULE="desktop.UserProfile"

LOGIN_REDIRECT_URL = "/"

PYLINTRC = get_desktop_root('.pylintrc')


############################################################
# Part 4: Installation of apps
############################################################

# Libraries are loaded and configured before the apps
appmanager.load_libs()
_lib_conf_modules = filter(None, [app.conf for app in appmanager.DESKTOP_LIBS])
conf.initialize(_lib_conf_modules)

appmanager.load_apps()
for app in appmanager.DESKTOP_APPS:
  INSTALLED_APPS.extend(app.django_apps)

logging.debug("Installed Django modules: %s" % ",".join(map(str, appmanager.DESKTOP_MODULES)))

# Load app configuration
_app_conf_modules = filter(None, [app.conf for app in appmanager.DESKTOP_APPS])
_app_conf_modules.append(desktop.conf)
conf.initialize(_app_conf_modules)

############################################################
# Part 4a: Django configuration that requires bound Desktop
# configs.
############################################################
# Configure database
DATABASE_ENGINE = desktop.conf.DATABASE.ENGINE.get()
DATABASE_NAME = desktop.conf.DATABASE.NAME.get()
DATABASE_USER = desktop.conf.DATABASE.USER.get()
DATABASE_PASSWORD = desktop.conf.DATABASE.PASSWORD.get()
DATABASE_HOST = desktop.conf.DATABASE.HOST.get()
DATABASE_PORT = desktop.conf.DATABASE.PORT.get()

# DB used for tests
TEST_DATABASE_NAME = get_desktop_root('desktop-test.db')

TIME_ZONE = desktop.conf.TIME_ZONE.get()
AUTHENTICATION_BACKENDS = (desktop.conf.AUTH.BACKEND.get(),)

EMAIL_HOST = desktop.conf.SMTP.HOST.get()
EMAIL_PORT = desktop.conf.SMTP.PORT.get()
EMAIL_HOST_USER = desktop.conf.SMTP.USER.get()
EMAIL_HOST_PASSWORD = desktop.conf.SMTP.PASSWORD.get()
EMAIL_USE_TLS = desktop.conf.SMTP.USE_TLS.get()
DEFAULT_FROM_EMAIL = desktop.conf.SMTP.DEFAULT_FROM.get()

############################################################



# Test configuration
TEST_RUNNER="django_nose.nose_runner.run_tests"

explicit_test_app = os.getenv("DESKTOP_TEST_ONLY")
if explicit_test_app:
  test_apps = [app.module.__name__ for app in appmanager.DESKTOP_APPS if app.name == explicit_test_app]
else:
  test_apps = [os.path.dirname(__file__)] + [app.module.__name__ for app in appmanager.DESKTOP_APPS ]
NOSE_ARGS = test_apps

# Depender configuration
DEPENDER_ROOT = get_desktop_root(".")

def prep_depender_config():
  yamls = [
    "../ext/thirdparty/js/mootools-core/package.yml",
    "../ext/thirdparty/js/mootools-more/package.yml",
    "../ext/thirdparty/js/art/package.yml",
    "../ext/thirdparty/js/mootools-touch/package.yml",
    "../ext/thirdparty/js/mootools-table/package.yml",
    "../ext/thirdparty/js/mootools-color/package.yml",
    "core/static/js/package.yml"
    ]
  scripts_json = [
    ("clientcide",  "../ext/thirdparty/js/clientcide/Source/scripts.json"),
    ("art-widgets",  "../ext/thirdparty/js/art-widgets/Source/scripts.json"),
    ("slick",  "../ext/thirdparty/js/art-widgets/assets/slick/Source/scripts.json"),
    ]


  for app in appmanager.DESKTOP_APPS:
    yamls.extend(app.depender_yamls)
    scripts_json.extend(app.depender_jsons)

  return ([ os.path.abspath(os.path.join(DEPENDER_ROOT, x)) for x in yamls ],
    [ (name, os.path.abspath(os.path.join(DEPENDER_ROOT, p))) for name, p in scripts_json ])

DEPENDER_PACKAGE_YMLS, DEPENDER_SCRIPTS_JSON = prep_depender_config()
DEPENDER_YUI_PATH = None
DEPENDER_COMPRESSOR = None
# Set to true to re-load all JS every time. (slowish)
DEPENDER_DEBUG = os.getenv("DESKTOP_DEPENDER_DEBUG", "0") not in ["0",""]
