#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.

import logging
import zipfile
import tempfile
import time

from django.shortcuts import render_to_response
from django.http import HttpResponse
from django.core.servers.basehttp import FileWrapper

from desktop.lib.django_util import render_json, render
from desktop.models import UserPreferences
from desktop import appmanager
import desktop.lib.log_buffer

LOG = logging.getLogger(__name__)

def log_view(request):
  """
  We have a log handler that retains the last X characters of log messages.
  If it is attached to the root logger, this view will display that history,
  otherwise it will report that it can't be found.
  """
  l = logging.getLogger()
  for h in l.handlers:
    if isinstance(h,desktop.lib.log_buffer.FixedBufferHandler):
      return render_to_response("logs.html", dict(log=[l for l in h.buf]))
  return render_to_response("logs.html", dict(log=["No logs found!"]))

def download_log_view(request):
  """
  Zip up the log buffer and then return as a file attachment.
  """
  l = logging.getLogger()
  for h in l.handlers:
    if isinstance(h,desktop.lib.log_buffer.FixedBufferHandler):
      try:
        # We want to avoid doing a '\n'.join of the entire log in memory
        # in case it is rather big. So we write it to a file line by line
        # and pass that file to zipfile, which might follow a more efficient path.
        tmp = tempfile.NamedTemporaryFile()
        log_tmp = tempfile.NamedTemporaryFile("w+t")
        for l in h.buf:
          log_tmp.write(l + '\n')
        # This is not just for show - w/out flush, we often get truncated logs
        log_tmp.flush()
        t = time.time()

        zip = zipfile.ZipFile(tmp, "w", zipfile.ZIP_DEFLATED)
        zip.write(log_tmp.name,"cloudera-desktop-logs/cloudera-desktop-%s.log" % t)
        zip.close()
        length = tmp.tell()

        # if we don't seek to start of file, no bytes will be written
        tmp.seek(0)
        wrapper = FileWrapper(tmp)
        response = HttpResponse(wrapper,content_type="application/zip")
        response['Content-Disposition'] = 'attachment; filename=cloudera-desktop-logs-%s.zip' % t
        response['Content-Length'] = length
        return response
      except Exception, e:
        logging.exception("Couldn't construct zip file to write logs to!")
        return log_view(request)

  return render_to_response("logs.html", dict(log=["No logs found!"]))

def prefs(request, key=None):
  """Get or set preferences."""
  if key is None:
    d = dict( (x.key, x.value) for x in UserPreferences.objects.filter(user=request.user))
    return render_json(d)
  else:
    if "set" in request.REQUEST:
      try:
        x = UserPreferences.objects.get(user=request.user, key=key)
      except UserPreferences.DoesNotExist:
        x = UserPreferences(user=request.user, key=key)
      x.value = request.REQUEST["set"]
      x.save()
      return render_json(True)
    if "delete" in request.REQUEST:
      try:
        x = UserPreferences.objects.get(user=request.user, key=key)
        x.delete()
        return render_json(True)
      except UserPreferences.DoesNotExist:
        return render_json(False)
    else:
      try:
        x = UserPreferences.objects.get(user=request.user, key=key)
        return render_json(x.value)
      except UserPreferences.DoesNotExist:
        return render_json(None)

def bootstrap(request):
  """Concatenates bootstrap.js files from all installed desktop apps."""

  # Has some None's for apps that don't have bootsraps.
  all_bootstraps = [ (app, app.get_bootstrap_file()) for app in appmanager.DESKTOP_APPS ]
  # Iterator over the streams.
  concatenated = [ "\n/* %s */\n%s" % (app.name, b.read()) for app, b in all_bootstraps if b is not None ]

  # HttpResponse can take an iteratable as the first argument, which
  # is what happens here.
  return HttpResponse(concatenated, mimetype='text/javascript')
bootstrap.login_notrequired = True

_status_bar_views = []
def register_status_bar_view(view):
  global _status_bar_views
  _status_bar_views.append(view)

def status_bar(request):
  """
  Concatenates multiple views together to build up a "status bar"/"status_bar".
  These views are registered using register_status_bar_view above.
  """
  global _status_bar_views
  resp = ""
  for view in _status_bar_views:
    try:
      r = view(request)
      if r.status_code == 200:
        resp += r.content
      else:
        LOG.warning("Failed to execute status_bar view %s" % view)
    except:
      LOG.exception("Failed to execute status_bar view %s" % view)
  return HttpResponse(resp)

def dump_config(request):
  # Note that this requires login (as do most apps).
  show_private = False
  if request.GET.get("private"):
    show_private = True

  return render("dump_config.mako", request, dict(show_private=show_private,
    top_level=desktop.lib.conf.GLOBAL_CONFIG, apps=appmanager.DESKTOP_MODULES))

def index(request):
  return render("index.mako", request, dict(
    feedback_url=desktop.conf.FEEDBACK_URL.get()
  ));
index.login_notrequired = True
