#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
"""Settings to configure your Hadoop cluster."""
from desktop.lib.conf import Config, UnspecifiedConfigSection, ConfigSection
import glob
import os
import logging

HADOOP_HOME = Config(
  key="hadoop_home",
  default=os.environ.get("HADOOP_HOME", "/usr/lib/hadoop-0.20"),
  help=("Path to directory holding hadoop libs - HADOOP_HOME in " +
        "hadoop parlance; defaults to environment variable, when" +
        "set.")
)

def hadoop_bin_from_hadoop_home():
  """Returns $HADOOP_HOME/bin/hadoop-0.20"""
  return os.path.join(HADOOP_HOME.get(), "bin/hadoop")

HADOOP_BIN = Config("hadoop_bin",
  help="Path to your Hadoop binary",
  dynamic_default=hadoop_bin_from_hadoop_home,
  type=str)

# TODO(philip): This will need more love for dealing with multiple clusters.
HADOOP_CONF_DIR = Config(
  key="hadoop_conf_dir",
  default=None,
  help="If set, directory to pass to hadoop_bin (from hadoop configuration) as the --config flag.",
)

def find_jar(desired_glob):
  def f():
    possibilities = glob.glob(os.path.join(HADOOP_HOME.get(), desired_glob))
    if len(possibilities) == 1:
      return possibilities[0]
    else:
      logging.warning("Trouble finding jars matching %s" % desired_glob)
      return None
  f.__doc__ = "Finds $HADOOP_HOME/%s" % desired_glob
  return f

def find_examples_jar():
  """
  Finds $HADOOP_HOME/hadoop-*-examples.jar
  """
  find_jar("hadoop-*-examples.jar")

HADOOP_EXAMPLES_JAR = Config(
  key="hadoop_examples_jar",
  dynamic_default=find_jar("hadoop-*-examples.jar"),
  help="Path to the hadoop-examples.jar (used for tests and jobdesigner setup)",
  type=str,
  private=True)

HADOOP_STREAMING_JAR = Config(
  key="hadoop_examples_jar",
  dynamic_default=find_jar(os.path.join("contrib", "streaming", "hadoop-*-streaming.jar")),
  help="Path to the hadoop-streaming.jar (used by jobdesigner)",
  type=str,
  private=True)

HADOOP_TEST_JAR = Config("hadoop_test_jar",
  help="[Used by testing code.] Path to hadoop-<version>-test.jar",
  dynamic_default=find_jar("hadoop-*-test.jar"),
  type=str,
  private=True)

HADOOP_PLUGIN_CLASSPATH = Config("hadoop_plugin_classpath",
  help="[Used only in testing code.] Path to the Hadoop plugin jar.",
  type=str,
  private=True)

HDFS_CLUSTERS = UnspecifiedConfigSection(
  "hdfs_clusters",
  help="One entry for each HDFS cluster",
  each=ConfigSection(
    help="Information about a single HDFS cluster",
    members=dict(
      NN_HOST=Config("namenode_host", help="IP for name node"),
      NN_THRIFT_PORT=Config("thrift_port", help="Thrift port for name node", default=9090,
                            type=int),
      NN_HDFS_PORT=Config("hdfs_port", help="Hadoop IPC port for the name node", default=8020,
                            type=int)
    )
  )
)

MR_CLUSTERS = UnspecifiedConfigSection(
  "mapred_clusters",
  help="One entry for each MapReduce cluster",
  each=ConfigSection(
    help="Information about a single MapReduce cluster",
    members=dict(
      JT_HOST=Config("jobtracker_host", help="IP for JobTracker"),
      JT_THRIFT_PORT=Config("thrift_port", help="Thrift port for JobTracker", default=9290,
                            type=int))))
