#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
"""
Tests for Hadoop FS.
"""
from nose.tools import assert_false, assert_true, assert_equals, assert_raises
from nose.plugins.attrib import attr
import posixfile
import random

from hadoop import mini_cluster
from hadoop.fs.exceptions import PermissionDeniedException

@attr('requires_hadoop')
def test_hadoopfs():
  """
  Minimal tests for a few basic file system operations.
  """
  cluster = mini_cluster.shared_cluster()
  try:
    fs = cluster.fs
    fs.setuser(cluster.superuser)
    f = fs.open("/fortest.txt", "w")
    try:
      f.write("hello")
      f.close()
      assert_equals("hello", fs.open("/fortest.txt").read())
      assert_equals(5, fs.stats("/fortest.txt")["size"])
      assert_true(fs.isfile("/fortest.txt"))
      assert_false(fs.isfile("/"))
      assert_true(fs.isdir("/"))
      assert_false(fs.isdir("/fortest.txt"))
    finally:
      fs.remove("/fortest.txt")
  finally:
    cluster.shutdown()

@attr('requires_hadoop')
def test_seek():
  """Test for DESKTOP-293 - ensure seek works in python2.4"""
  cluster = mini_cluster.shared_cluster()
  try:
    fs = cluster.fs
    fs.setuser(cluster.superuser)
    f = fs.open("/fortest.txt", "w")
    try:
      f.write("hello")
      f.close()

      f = fs.open("/fortest.txt", "r")
      f.seek(0, posixfile.SEEK_SET)
      assert_equals("he", f.read(2))
      f.seek(1, posixfile.SEEK_SET)
      assert_equals("el", f.read(2))
      f.seek(-1, posixfile.SEEK_END)
      assert_equals("o", f.read())
      f.seek(0, posixfile.SEEK_SET)
      f.seek(2, posixfile.SEEK_CUR)
      assert_equals("ll", f.read(2))
    finally:
      fs.remove("/fortest.txt")
  finally:
    cluster.shutdown()

@attr('requires_hadoop')
def test_seek_across_blocks():
  """Makes a file with a lot of blocks, seeks around"""
  cluster = mini_cluster.shared_cluster()
  try:
    fs = cluster.fs
    fs.setuser(cluster.superuser)
    f = fs.open("/fortest-blocks.txt", "w", block_size=1024)
    try:
      data = "abcdefghijklmnopqrstuvwxyz" * 3000
      f.write(data)
      f.close()

      for i in xrange(1, 10):
        f = fs.open("/fortest-blocks.txt", "r")

        for j in xrange(1, 100):
          offset = random.randint(0, len(data) - 1)
          f.seek(offset, posixfile.SEEK_SET)
          assert_equals(data[offset:offset+50], f.read(50))
        f.close()

    finally:
      fs.remove("/fortest-blocks.txt")
  finally:
    cluster.shutdown()


@attr('requires_hadoop')
def test_exceptions():
  """
  Tests that appropriate exceptions are raised.
  """
  cluster = mini_cluster.shared_cluster()
  try:
    fs = cluster.fs
    fs.setuser(cluster.superuser)
    f = fs.open("/for_exception_test.txt", "w")
    f.write("foo")
    f.close()
    fs.chmod("/for_exception_test.txt", 0400)
    fs.setuser("notsuperuser")
    f = fs.open("/for_exception_test.txt")
    # Arguably, this should have thrown already, at open, but
    # we throw the exception lazily, when getting block locations.
    assert_raises(PermissionDeniedException, f.read)

    assert_raises(IOError, fs.open, "/test/doesnotexist.txt")
  finally:
    cluster.shutdown()

@attr('requires_hadoop')
def test_two_files_open():
  """
  See DESKTOP-510.  There was a bug where you couldn't open two files at
  the same time.  It boils down to a close_fds=True issue.  If this doesn't
  hang, all is good.
  """
  cluster = mini_cluster.shared_cluster()
  try:
    fs = cluster.fs
    fs.setuser(cluster.superuser)
    f1 = fs.open("/test_one.txt", "w")
    f2 = fs.open("/test_two.txt", "w")
    f1.write("foo")
    f2.write("bar")
    f1.close()
    f2.close()
    # This should work, not hang, etc.
  finally:
    cluster.shutdown()
