#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
#
# Django-side implementation of the JobTracker plugin interface

import random
import time

from desktop.lib import thrift_util
from desktop.lib.thrift_util import fixup_enums

from hadoop.api.jobtracker import Jobtracker
from hadoop.api.jobtracker.ttypes import *
from hadoop.api.common.ttypes import RequestContext

VALID_TASK_STATES = set(["succeeded", "failed", "running", "pending", "killed"])
VALID_TASK_TYPES = set(["map", "reduce", "job_cleanup", "job_setup"])

# timeout (seconds) for thrift calls to jobtracker
JT_THRIFT_TIMEOUT=15

class AbstractJobTracker(object):

  def thriftjobid_from_string(self, jobid):
    """The jobid looks like this: job_201001301455_0001"""
    _, tid, jid = jobid.split("_")
    return ThriftJobID(tid, int(jid), jobid)

  def thriftattemptid_from_string(self, attemptid):
    """The attemptid looks like this: attempt_201001301455_0001_m_000007_0"""
    _, meat = attemptid.split('_', maxsplit=1)
    taskid, aid = meat.rsplit('_', maxsplit=1)
    return ThriftTaskAttemptID(thrifttaskid_from_string(taskid), int(aid))

  def thrifttaskid_from_string(self, taskid_str):
    """The taskid_str looks like this: task_201001301455_0001_m_000007"""
    TASK_TYPE_MAP = { "m" : ThriftTaskType.MAP,
                      "r" : ThriftTaskType.REDUCE }
    _, tracker, jid, tasktype, tid = taskid_str.split('_')
    return ThriftTaskID(ThriftJobID(tracker, int(jid)),
                        TASK_TYPE_MAP[tasktype],
                        int(tid))

  def _fixup_tasktracker(self, tracker):
    """
    Adds string representations of enums to a ThirftTaskTrackerStatus
    """
    if not tracker.taskReports:
      return tracker
    for tts in tracker.taskReports:
      fixup_enums(tts.taskID.taskID, {"taskType":ThriftTaskType})
      self._fixup_taskstatus(tts)


  def _fixup_taskstatus(self, taskstatus):
    """
    Adds string representations of enums to a ThriftTaskInProgress
    """
    fixup_enums(taskstatus, {"state":ThriftTaskState, "phase":ThriftTaskPhase})
    fixup_enums(taskstatus.taskID.taskID, {"taskType":ThriftTaskType})

  def _fixup_task_state(self, tip):
    """
    Determine overall state of a task.
    """
    if tip.complete:
      tip.state = "succeeded"
    elif tip.failed:
      if not tip.execStartTime:
        tip.state = "killed"
      else:
        tip.state = "failed"
    elif tip.execStartTime:
      tip.state = "running"
    else:
      tip.state = "pending"

    global VALID_TASK_STATES
    assert tip.state in VALID_TASK_STATES

  def _fixup_job(self, job):
    """
    Run through all the enums in a job and add their string variant
    """
    fixup_enums(job.status, {"runState":ThriftJobState, "priority":ThriftJobPriority})
    fixup_enums(job, {"priority":ThriftJobPriority})
    if job.tasks is not None:
      for tip in job.tasks.tasks:
        fixup_enums(tip.taskID, {"taskType":ThriftTaskType})
        self._fixup_task_state(tip)
        for taskstatus in tip.taskStatuses.values():
          self._fixup_taskstatus(taskstatus)

  def _fixup_task_in_progress(self, tip):
    fixup_enums(tip.taskID, {"taskType": ThriftTaskType})
    self._fixup_task_state(tip)
    for taskstatus in tip.taskStatuses.values():
      self._fixup_taskstatus(taskstatus)


DEFAULT_USER = "webui"
DEFAULT_GROUPS = ["webui"]

class LiveJobTracker(AbstractJobTracker):
  """
  Connects to a JobTracker over our Thrift interface.
  At the moment, doesn't handle Thrift errors gracefully.

  In particular, if Thrift returns None for anything, this will throw.
  """

  def __init__(self, host, thrift_port):
    self.client = thrift_util.get_client(
      Jobtracker.Client, host, thrift_port, service_name="Hadoop MR JobTracker",
      timeout_seconds=JT_THRIFT_TIMEOUT)
    self.host = host
    self.thrift_port = thrift_port
    self.request_context = RequestContext()
    self.setuser(DEFAULT_USER, DEFAULT_GROUPS)

  def setuser(self, user, groups=None):
    # Hadoop UGI *must* have at least one group, so we mirror
    # the username as a group if not specified
    if not groups:
      groups = [user]
    if not self.request_context.confOptions:
      self.request_context.confOptions = {}
    self.ugi = ",".join([user] + groups)
    self.request_context.confOptions['hadoop.job.ugi'] = self.ugi

  def queues(self):
    """
    Returns a ThriftJobQueueList
    """
    qs = self.client.getQueues(None)
    return qs

  def cluster_status(self):
    """
    Returns a ThriftClusterStatus
    """
    cs = self.client.getClusterStatus(None)
    fixup_enums(cs, {"state":JobTrackerState})
    return cs

  def runtime_info(self):
    """
    Returns a RuntimeInfo
    """
    return self.client.getRuntimeInfo(None)

  def all_task_trackers(self):
    """
    Returns a ThriftTaskTrackerStatusList
    """
    tts = self.client.getAllTrackers(None)
    for tracker in tts.trackers:
      self._fixup_tasktracker(tracker)
    return tts

  def active_trackers(self):
    """
    Returns a ThriftTaskTrackerStatusList
    """
    tts = self.client.getActiveTrackers(None)
    for tracker in tts.trackers:
      self._fixup_tasktracker(tracker)
    return tts

  def blacklisted_trackers(self):
    """
    Returns a ThriftTaskTrackerStatusList
    """
    tts = self.client.getBlacklistedTrackers(None)
    for tracker in tts.trackers:
      self._fixup_tasktracker(tracker)
    return tts

  def task_tracker(self, name):
    """
    Returns a ThriftTaskTrackerStatus or None
    """
    tracker = self.client.getTracker(None, name)
    if not tracker:
      return None
    self._fixup_tasktracker(tracker)
    return tracker

  def get_job(self, jobid):
    """
    Returns a ThriftJobInProgress (including task info)
    """
    try:
      job = self.client.getJob(None, jobid)
    except JobNotFoundException, e:
      e.response_data = dict(code="JT_JOB_NOT_FOUND", message="Could not find job %s on JobTracker." % jobid.asString, data=jobid)
      raise
    self._fixup_job(job)
    return job

  def running_jobs(self):
    """
    Returns a ThriftJobList (does not include task info)
    """
    joblist = self.client.getRunningJobs(None)
    for job in joblist.jobs:
      self._fixup_job(job)
    return joblist

  def completed_jobs(self):
    """
    Returns a ThriftJobList (does not include task info)
    """
    joblist = self.client.getCompletedJobs(None)
    for job in joblist.jobs:
      self._fixup_job(job)
    return joblist

  def failed_jobs(self):
    """
    Returns a ThriftJobList (does not include task info)
    """
    joblist = self.client.getFailedJobs(None)
    for job in joblist.jobs:
      self._fixup_job(job)
    return joblist

  def all_jobs(self):
    """
    Returns a ThriftJobList (does not include task info)
    """
    joblist = self.client.getAllJobs(None)
    for job in joblist.jobs:
      self._fixup_job(job)
    return joblist

  def get_job_count_by_user(self, user):
    """
    Returns a ThriftUserJobCounts.
    """
    return self.client.getUserJobCounts(self.request_context, user)

  def get_job_counters(self, jobid):
    """
    Returns a ThriftGroupList
    """
    return self.client.getJobCounters(None, jobid)

  def get_job_counter_rollups(self, jobid):
    """
    Returns a ThriftGroupList
    """
    return self.client.getJobCounterRollups(None, jobid)


  def get_task_list(self, jobid, task_types, task_states, task_text, count, offset):
    """
    Return a ThriftTaskInProgressList
    """
    # Translate task_states and task_types to their thrift variants
    ttask_types = [ ThriftTaskType._NAMES_TO_VALUES[x.upper()] for x in task_types ]
    ttask_states = [ ThriftTaskQueryState._NAMES_TO_VALUES[x.upper()] for x in task_states ]
    tip_list = self.client.getTaskList(
          self.request_context, jobid, ttask_types, ttask_states, task_text, count, offset)

    for tip in tip_list.tasks:
      self._fixup_task_in_progress(tip)
    return tip_list

  def get_task(self, jobid, taskid):
    """Return a ThriftTaskInProgress"""
    try:
      tip = self.client.getTask(jobid, taskid)
    except JobNotFoundException, e:
      e.response_data = dict(code="JT_JOB_NOT_FOUND",
                             message="Could not find job %s on JobTracker." % (jobid.asString,),
                             data=jobid)
      raise e
    except TaskNotFoundException, e:
      e.response_data = dict(code="JT_TASK_NOT_FOUND",
                             message="Could not find task %s on JobTracker." % (taskid.asString,),
                             data=taskid)
      raise e
    self._fixup_task_in_progress(tip)
    return tip

  def get_current_time(self):
    """
    Returns an integer timestamp
    """
    return self.client.getCurrentTime(None)

  def get_job_xml(self, jobid):
    """
    Returns a string representation of the job XML
    """
    return self.client.getJobConfXML(None, jobid)

  def kill_job(self, jobid):
    """
    Kill a job
    """
    return self.client.killJob(self.request_context, jobid)

  def kill_task_attempt(self, attemptid):
    """
    Kill a task attempt
    """
    return self.client.killTaskAttempt(self.request_context, attemptid)

  def set_job_priority(self, jobid, priority):
    """
    Set a job's priority
    """
    return self.client.setJobPriority(None, jobid, priority)

class MockJobTracker(AbstractJobTracker):
  """
  Implements all the goodness of a JobTracker with none of the
  fat of an actual Thrift session.
  """

  def __init__(self, host, port):
    self.host = host
    self.port = port

  def _fake_task_tracker(self, id=1):
    ret = ThriftTaskTrackerStatus(trackerName="tracker%d" % id,
                                  host="localhost",
                                  httpPort=9090,
                                  failureCount=1,
                                  taskReports=[],
                                  lastSeen=654321,
                                  maxMapTasks=10,
                                  maxReduceTasks=20,
                                  totalVirtualMemory=1000000,
                                  reservedTotalMemory=100000,
                                  reservedPhysicalMemory=1000,
                                  availableSpace=90000,
                                  mapCount=3,
                                  reduceCount=5)
    return ret


  def _fake_active_trackers(self):
    return [self._fake_task_tracker(id) for id in xrange(3)]

  def _fake_blacklisted_trackers(self):
    return [self._fake_task_tracker(id) for id in xrange(3, 5)]

  def _fake_cluster_status(self):
    cs = ThriftClusterStatus(numActiveTrackers=1,
                               activeTrackerNames=['tracker0', 'tracker1', 'tracker2'],
                               blacklistedTrackerNames=['tracker3', 'tracker4'],
                               numBlacklistedTrackers=0,
                               numExcludedNodes=0,
                               taskTrackerExpiryInterval=100,
                               mapTasks=3,
                               reduceTasks=3,
                               maxMapTasks=100,
                               maxReduceTasks=100,
                               state=JobTrackerState.RUNNING,
                               usedMemory=1000000,
                               maxMemory=6400000,
                               identifier="dummyidentifier",
                               hostname="localhost",
                               httpPort=4321,
                               startTime= 10
                               )
    cs.stateAsString = "RUNNING"
    return cs

  def _fake_task_status(self, jobid, taskid, type):
    startTime = random.randint(0, 100000000)
    def get_random_enum(obj):
      return getattr(obj, random.choice(obj._VALUES_TO_NAMES.values()))

    phase = get_random_enum(ThriftTaskPhase)
    state = get_random_enum(ThriftTaskState)
    ret = ThriftTaskStatus(progress=random.random(),
                           taskID=ThriftTaskID(jobid, type, taskid,
                                               "%s_%s"%(jobid.asString, taskid)),
                           state=state,
                           startTime=startTime,
                           finishTime=startTime+random.randint(0, 100000),
                           outputSize=random.randint(0, 1000000),
                           phase=phase,
                           taskTracker="tracker%d" % random.randint(0, 1),
                           counters=None)
    ret.phaseAsString = ThriftTaskPhase._VALUES_TO_NAMES[phase]
    ret.stateAsString = ThriftTaskState._VALUES_TO_NAMES[state]
    return ret

  def _fake_task_in_progress(self, jobid, taskid, type):
    # Prime numbers are cool.
    random.seed((2**jobid.jobID)+(3**taskid))
    taskID = ThriftTaskID(jobid, type, taskid, "%s_%s"%(jobid.asString, taskid))
    taskID.typeAsString = ThriftTaskType._VALUES_TO_NAMES[type]
    numAttempts=random.randint(1, 5)
    tasks = [ThriftTaskAttemptID(taskID, x, "%s_%s" % (taskID.asString, x))
             for x in xrange(numAttempts)]
    statuses = {}
    for t in tasks:
      statuses[t.asString] = self._fake_task_status(jobid, taskid, type)
    ret = ThriftTaskInProgress(execStartTime=0,
                               execFinishTime=10,
                               progress=random.random(),
                               startTime=0,
                               failed=False,
                               complete=False,
                               taskID=taskID,
                               tasks=tasks,
                               taskStatuses=statuses,
                               taskDiagnosticData=None,
                               counters=None)
    return ret

  def _fake_job_status(self):
    ret = ThriftJobStatus()
    ret.mapProgress, ret.reduceProgress = random.random(), random.random()
    ret.cleanupProgress, ret.setupProgress = random.random(), random.random()
    ret.startTime = random.randint(100000000, 2000000000)
    return ret

  def _fake_job_profile(self, user="henry", jobid=None):
    ret = ThriftJobProfile()
    ret.user = user
    ret.queuename = "randomqueue"
    ret.name = "some job %d" % jobid.jobID
    ret.jobFile = "file:///dummy%s.xml" % jobid.asString
    ret.jobID = jobid
    return ret

  def _fake_running_job(self, user="henry", jobid=0):
    random.seed(jobid)
    ret = ThriftJobInProgress()
    ret.jobID = ThriftJobID("dummyjobid", jobid, "dummyjobid_%d" % jobid)
    ret.status = self._fake_job_status()
    ret.profile = self._fake_job_profile(user=user, jobid=ret.jobID)
    ret.desiredReduces = random.randint(1, 10)
    ret.finishedReduces = random.randint(0, ret.desiredReduces)
    ret.desiredMaps = random.randint(1, 10)
    ret.finishedMaps = random.randint(0, ret.desiredMaps)
    ret.priority = random.randint(1, 4)
    ret.status.mapProgress = ret.finishedMaps / ret.desiredMaps
    ret.status.reduceProgress = ret.finishedReduces / ret.desiredReduces
    ret.tasks = [self._fake_task_in_progress(ret.jobID, x, 0)
                  for x in xrange(ret.desiredMaps)]
    ret.tasks += [self._fake_task_in_progress(ret.jobID, x, 1)
                  for x in xrange(ret.desiredReduces)]
    ret.startTime = random.randint(10000000, 99999999)
    ret.launchTime = ret.startTime + 6000
    ret.finishTime = ret.launchTime + random.randint(1, 30*60)*1000
    return ret

  def _fake_queues(self):
    l = ThriftJobQueueList([ThriftJobQueueInfo(queueName=x)
                            for x in ["queue1", "queue2", "randomqueue"] ] )
    return l

  def running_jobs(self, filter=None):
    ret = [ self._fake_running_job(*i) for i in zip(['henry', 'philip', 'todd', 'henry'], xrange(4))]
    return ThriftJobList(ret)

  def completed_jobs(self, filter=None):
    ret = [ self._fake_running_job(*i) for i in zip(['henry', 'philip', 'todd', 'henry'], xrange(4, 8))]
    return ThriftJobList(ret)

  def failed_jobs(self, filter=None):
    ret = [ self._fake_running_job(*i) for i in zip(['henry', 'philip', 'todd', 'henry'], xrange(8, 12))]
    return ThriftJobList(ret)

  def all_jobs(self, filter=None):
    ret = [ self._fake_running_job(*i) for i in zip(['henry', 'philip', 'todd', 'henry']*3, xrange(12)) ]
    return ThriftJobList(ret)

  def queues(self):
    return self._fake_queues()

  def cluster_status(self, ):
    return self._fake_cluster_status()

  def tracker_name(self):
    return "dummyTrackerName"

  def all_task_trackers(self):
    return ThriftTaskTrackerStatusList(trackers=self._fake_active_trackers() + self._fake_blacklisted_trackers())

  def active_trackers(self):
    return ThriftTaskTrackerStatusList(trackers=self._fake_active_trackers())

  def blacklisted_trackers(self):
    return self._fake_blacklisted_trackers()
    return ThriftTaskTrackerStatusList(trackers=self._fake_blacklisted_trackers())

  def task_tracker(self, name):
    """
    Returns task tracker with name same as name.
    """
    for t in self.active_trackers():
      if t.trackerName == name:
        return t
    for t in self.blackisted_trackers():
      if t.trackerName == name:
        return t
    return None

  def get_current_time(self):
    return int(time.time()*1000.0)

  def get_job_xml(self, jobid):
    """
    Returns a string representation of the job XML
    """
    return """<?xml version="1.0" encoding="UTF-8" standalone="no"?><configuration>
<property><name>fs.s3n.impl</name><value>org.apache.hadoop.fs.s3native.NativeS3FileSystem</value></property>
<property><name>dfs.datanode.plugins</name><value>org.apache.hadoop.thriftfs.DatanodePlugin</value></property>
<property><name>mapred.task.cache.levels</name><value>2</value></property>
<property><name>mapred.thrift.jobtracker.address</name><value>0.0.0.0:9091</value></property>
<property><name>hadoop.tmp.dir</name><value>/tmp/tmp-hadoop</value></property>
<property><name>hadoop.native.lib</name><value>true</value></property>
<property><name>map.sort.class</name><value>org.apache.hadoop.util.QuickSort</value></property>
<property><name>dfs.namenode.decommission.nodes.per.interval</name><value>5</value></property>
<property><name>dfs.https.need.client.auth</name><value>false</value></property>
<property><name>ipc.client.idlethreshold</name><value>4000</value></property>
<property><name>mapred.system.dir</name><value>${hadoop.tmp.dir}/mapred/system</value></property>
<property><name>mapred.job.tracker.persist.jobstatus.hours</name><value>0</value></property>
<property><name>dfs.datanode.address</name><value>0.0.0.0:50010</value></property>
<property><name>dfs.namenode.logging.level</name><value>info</value></property>
<property><name>io.skip.checksum.errors</name><value>false</value></property>
<property><name>fs.default.name</name><value>hdfs://localhost:9000/</value></property>
<property><name>mapred.reducer.new-api</name><value>true</value></property>
<property><name>dfs.thrift.address</name><value>0.0.0.0:9090</value></property>
<property><name>mapred.child.tmp</name><value>./tmp</value></property>
<property><name>dfs.safemode.threshold.pct</name><value>0.999f</value></property>
<property><name>mapred.skip.reduce.max.skip.groups</name><value>0</value></property>
<property><name>dfs.namenode.handler.count</name><value>10</value></property>
<property><name>dfs.blockreport.initialDelay</name><value>0</value></property>
<property><name>mapred.jobtracker.instrumentation</name><value>org.apache.hadoop.mapred.JobTrackerMetricsInst</value></property>
</configuration>
"""
