#!/usr/bin/env python
# (c) Copyright 2010 Cloudera, Inc. All rights reserved.
#
# A Python-side driver for MiniHadoopClusterManager
# 
# See README.testing for hints on how to use this,
# and also look for other examples.

import atexit
import subprocess
import os
import logging
import sys
import signal
import shutil
import socket
import time
import tempfile
import simplejson
import lxml.etree

from hadoop.fs.hadoopfs import HadoopFileSystem
from hadoop.job_tracker import LiveJobTracker
import hadoop.cluster
import health.clusters
import desktop.lib.fsmanager

# Starts mini cluster suspended until a debugger attaches to it.
DEBUG_HADOOP=False
# Redirects mini cluster stderr to stderr.  (Default is to put it in a file.)
USE_STDERR=os.environ.get("MINI_CLUSTER_USE_STDERR", False)
# Whether to clean up temp dir at exit
CLEANUP_TMP_DIR=os.environ.get("MINI_CLUSTER_CLEANUP", True)
# How long to wait for cluster to start up.  (seconds)
MAX_CLUSTER_STARTUP_TIME = 120.0

LOGGER=logging.getLogger(__name__)

def _find_unused_port():
  """
  Finds a port that's available.
  Unfortunately, this port may not be available by the time
  the subprocess uses it, but this generally works.
  """
  sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM, 0)
  sock.bind(('127.0.0.1', 0))
  sock.listen(socket.SOMAXCONN)
  _, port = sock.getsockname()
  sock.close()
  return port

class MiniHadoopCluster(object):
  """
  Manages the invocation of a MiniHadoopClusterManager from Python.
  """
  def __init__(self, num_datanodes=1, num_tasktrackers=1):
    # These are cached
    self._jt, self._fs = None, None
    self.num_datanodes = num_datanodes
    self.num_tasktrackers = num_tasktrackers

  def start(self):
    """
    Start a cluster as a subprocess.
    """
    self.tmpdir = tempfile.mkdtemp()

    def tmppath(filename):
      """Creates paths in tmpdir."""
      return os.path.join(self.tmpdir, filename)

    LOGGER.info("Using temporary directory: %s" % self.tmpdir)

    in_conf_dir = tmppath("in-conf")
    os.mkdir(in_conf_dir)
    f = file(os.path.join(in_conf_dir, "hadoop-metrics.properties"), "w")
    try:
      f.write("""
dfs.class=org.apache.hadoop.metrics.spi.NoEmitMetricsContext
mapred.class=org.apache.hadoop.metrics.spi.NoEmitMetricsContext
jvm.class=org.apache.hadoop.metrics.spi.NoEmitMetricsContext
rpc.class=org.apache.hadoop.metrics.spi.NoEmitMetricsContext
""")
    finally:
      f.close()
  
    details_file = file(tmppath("details.json"), "w+")
    try:
      args = [ hadoop.conf.HADOOP_BIN.get(), "jar",
        hadoop.conf.HADOOP_TEST_JAR.get(),
        "minicluster",
        "-writeConfig", tmppath("config.xml"), 
        "-writeDetails", tmppath("details.json"),
        "-datanodes", str(self.num_datanodes),
        "-tasktrackers", str(self.num_tasktrackers),
        "-useloopbackhosts",
        "-D", "hadoop.tmp.dir=%s" % self.tmpdir,
        "-D", "mapred.local.dir=%s/mapred/local" % self.tmpdir,
        "-D", "mapred.system.dir=/mapred/system",
        "-D", "mapred.temp.dir=/mapred/temp",
        "-D", "dfs.namenode.plugins=org.apache.hadoop.thriftfs.NamenodePlugin",
        "-D", "dfs.datanode.plugins=org.apache.hadoop.thriftfs.DatanodePlugin",
        "-D", "mapred.jobtracker.plugins=org.apache.hadoop.thriftfs.ThriftJobTrackerPlugin",
        "-D", "jobclient.completion.poll.interval=100",
        "-D", "jobclient.progress.monitor.poll.interval=100",
        # We bind the NN's thrift interface to a port we find here.
        # This is suboptimal, since there's a race.  Alas, if we don't
        # do this here, the datanodes fail to discover the namenode's thrift
        # address, and there's a race there
        "-D", "dfs.thrift.address=localhost:%d" % _find_unused_port(),
        "-D", "jobtracker.thrift.address=localhost:%d" % _find_unused_port(),
        # Jobs realize they have finished faster with this timeout.
        "-D", "jobclient.completion.poll.interval=50",
      ]
      env = os.environ.copy()
      env["HADOOP_CONF_DIR"] = in_conf_dir
      env["HADOOP_OPTS"] = "-Dtest.build.data=%s" % self.tmpdir
      env["HADOOP_CLASSPATH"] = hadoop.conf.HADOOP_PLUGIN_CLASSPATH.get()
      env["HADOOP_HEAPSIZE"] = "128"
      env["HADOOP_HOME"] = hadoop.conf.HADOOP_HOME.get()
      # Wait for the debugger to attach
      if DEBUG_HADOOP:
        env["HADOOP_OPTS"] = env.get("HADOOP_OPTS", "") + " -Xdebug -Xrunjdwp:transport=dt_socket,server=y,suspend=y,address=9999"
        
      if USE_STDERR:
        stderr=sys.stderr
      else:
        stderr=file(tmppath("stderr"), "w")
      LOGGER.debug("Starting minicluster: %s env: %s" % (repr(args), repr(env)))
      self.clusterproc = subprocess.Popen(
        args=args,
        stdout=file(tmppath("stdout"), "w"),
        stderr=stderr,
        env=env)

      details = {}
      start = time.time()
      # We consider the cluster started when the details file parses correct JSON.
      # MiniHadoopCluster currently writes the details file last, and this depends
      # on that.
      while not details:
        try:
          details_file.seek(0)
          details = simplejson.load(details_file)
        except ValueError:
          pass
        if self.clusterproc.poll() is not None or (not DEBUG_HADOOP and (time.time() - start) > MAX_CLUSTER_STARTUP_TIME):
          LOGGER.debug("stdout:" + file(tmppath("stdout")).read())
          if not USE_STDERR:
            LOGGER.debug("stderr:" + file(tmppath("stderr")).read())
          if self.clusterproc.poll() is not None:
            os.kill(self.clusterproc.pid, signal.SIGKILL)
          raise Exception("Cluster process quit or is taking too long to start.  Aborting.")
    finally:
      details_file.close()

    # Place all the details as attributes on self.
    for k, v in details.iteritems():
      setattr(self, k, v)

    # Parse the configuration using XPath and place into self.config.
    config = lxml.etree.parse(tmppath("config.xml"))
    self.config = dict( (property.find("./name").text, property.find("./value").text) 
      for property in config.xpath("/configuration/property"))

    # Write out Hadoop-style configuration directory, 
    # which can, in turn, be used for /bin/hadoop.
    self.config_dir = tmppath("conf")
    os.mkdir(self.config_dir)
    write_config(self.config, tmppath("conf/core-site.xml"), 
      ["fs.default.name", "jobclient.completion.poll.interval"])
    write_config(self.config, tmppath("conf/hdfs-site.xml"), ["fs.default.name"])
    # mapred.job.tracker isn't written out into self.config, so we fill
    # that one out more manually.
    write_config({ "mapred.job.tracker": "localhost:%d" % self.jobtracker_port },
      tmppath("conf/mapred-site.xml"), ["mapred.job.tracker"])

  def stop(self):
    """
    Kills the cluster ungracefully.
    """
    pid = self.clusterproc.pid
    os.kill(pid, signal.SIGKILL)
    self.clusterproc.wait()

    if CLEANUP_TMP_DIR:
      logging.info("Cleaning up self.tmpdir.  Use $CLEANUP_TMP_DIR to avoid.")
      shutil.rmtree(self.tmpdir)

  @property
  def fs(self):
    """Creates a HadoopFileSystem object configured for this cluster."""
    if self._fs is None:
      self._fs = HadoopFileSystem("localhost", 
        thrift_port=self.namenode_thrift_port,
        hdfs_port=self.namenode_port,
        hadoop_bin_path=hadoop.conf.HADOOP_BIN.get())
    return self._fs

  @property
  def jt(self):
    """Creates a LiveJobTracker object configured for this cluster."""
    if self._jt is None:
      self._jt = LiveJobTracker("localhost", self.jobtracker_thrift_port)
    return self._jt

  @property
  def superuser(self):
    """
    Returns the "superuser" of this cluster.  
    
    This is essentially the user that the cluster was started
    with.
    """
    return self.config["hadoop.job.ugi"].split(",")[0]

  @property
  def namenode_thrift_port(self):
    """
    Return the namenode thrift port.
    """
    _, port = self.config["dfs.thrift.address"].split(":")
    return int(port)

  @property
  def jobtracker_thrift_port(self):
    """
    Return the jobtracker thrift port.
    """
    _, port = self.config["jobtracker.thrift.address"].split(":")
    return int(port)

  def dump_ini(self, fd=sys.stdout):
    """
    Dumps an ini-style configuration suitable for configuring desktop
    to talk to this cluster.
    TODO(todd) eventually this should use config framework 'writeback'
    support

    @param fd: a file-like writable object
    """
    print >>fd, "[hadoop]"
    print >>fd, "[[hdfs_clusters]]"
    print >>fd, "[[[default]]]"
    print >>fd, "thrift_port=%d" % self.namenode_thrift_port
    print >>fd, "[[mapred_clusters]]"
    print >>fd, "[[[default]]]"
    print >>fd, "thrift_port=%d" % self.jobtracker_thrift_port


# Shared global cluster returned by shared_cluster context manager.
_shared_cluster = None

def shared_cluster(conf=False):
  """
  Use a shared cluster that is initialized on demand,
  and that is torn down at process exit.

  If conf is True, then configuration is updated to
  reference the cluster, and relevant caches are cleared.

  Returns a lambda which must be called when you are
  done with the shared cluster.
  """
  cluster = shared_cluster_internal()
  closers = [ ]
  if conf:
    closers.extend([
      hadoop.conf.HDFS_CLUSTERS["default"].NN_HOST.set_for_testing("localhost"),
      hadoop.conf.HDFS_CLUSTERS["default"].NN_THRIFT_PORT.set_for_testing(cluster.fs.thrift_port),
      hadoop.conf.HDFS_CLUSTERS["default"].NN_HDFS_PORT.set_for_testing(cluster.namenode_port),
      hadoop.conf.MR_CLUSTERS["default"].JT_HOST.set_for_testing("localhost"),
      hadoop.conf.MR_CLUSTERS["default"].JT_THRIFT_PORT.set_for_testing(cluster.jt.thrift_port),
    ])
    # Clear the caches
    # This is djanky (that's django for "janky").
    # Caches are tricky w.r.t. to to testing;
    # perhaps there are better patterns?
    desktop.lib.fsmanager.reset()
    old = hadoop.cluster.clear_caches()
    old_clusters = health.clusters.Cluster.CLUSTERS
    health.clusters.Cluster.CLUSTERS = None

  def finish():
    if conf:
      hadoop.cluster.restore_caches(old)
      health.clusters.Cluster.CLUSTERS = old_clusters
    for x in closers:
      x()

  # We don't run the cluster's real stop method,
  # because a shared cluster should be shutdown at 
  # exit.
  cluster.shutdown = finish
  return cluster

def write_config(config, path, variables):
  """
  Minimal utility to write Hadoop-style configuration
  from a configuration map (config), into a new file
  called path.
  """
  f = file(path, "w")
  try:
    f.write("""<?xml version="1.0"?>
<?xml-stylesheet type="text/xsl" href="configuration.xsl"?>
<configuration>
""")
    for name in variables:
      value = config[name]
      f.write("<property>\n")
      f.write("  <name>%s</name>\n" % name)
      f.write("  <value>%s</value>\n" % value)
      f.write("</property>\n")
    f.write("</configuration>\n")
  finally:
    f.close()

def shared_cluster_internal():
  """
  Manages _shared_cluster.
  """
  global _shared_cluster
  if _shared_cluster is None:
    _shared_cluster = MiniHadoopCluster()
    _shared_cluster.start()
    atexit.register(_shared_cluster.stop)
  return _shared_cluster

if __name__ == '__main__':
  """
  It's poor form to write tests for tests (the world-wide stack
  overflow exception), so this merely tries the code.
  """
  logging.basicConfig(level=logging.DEBUG)
  import desktop
  desktop.lib.conf.initialize([hadoop.conf])

  if True:
    cluster = MiniHadoopCluster(num_datanodes=5, num_tasktrackers=5)
    cluster.start()
    print cluster.namenode_port
    print cluster.jobtracker_port
    print cluster.config.get("dfs.thrift.address")
    cluster.dump_ini(sys.stdout)

    from IPython.Shell import IPShellEmbed
    IPShellEmbed()()
    cluster.stop()
